"""
File Context Analyzer Service.

Analyzes individual files to determine their context (production, test, example, etc.)
for false positive prevention and context-aware analysis.
"""

import re
from pathlib import Path
from typing import Dict, List, Optional, Tuple
import structlog

from warden.analysis.domain.file_context import (
    FileContext,
    FileContextInfo,
    ContextWeights,
)
from warden.analysis.domain.project_context import ProjectContext

logger = structlog.get_logger()


class FileContextAnalyzer:
    """
    Analyzes file context for false positive prevention.

    Part of the PRE-ANALYSIS phase for context detection.
    Supports optional LLM enhancement for ambiguous cases.
    """

    def __init__(
        self,
        project_context: Optional[ProjectContext] = None,
        llm_analyzer: Optional['LlmContextAnalyzer'] = None,
    ) -> None:
        """
        Initialize analyzer.

        Args:
            project_context: Optional project context for better detection
            llm_analyzer: Optional LLM analyzer for enhanced detection
        """
        self.project_context = project_context
        self.llm_analyzer = llm_analyzer
        self.path_patterns = self._compile_path_patterns()
        self.content_patterns = self._compile_content_patterns()

    def analyze_file(self, file_path: Path) -> FileContextInfo:
        """
        Analyze a single file to determine its context (sync version).

        Args:
            file_path: Path to the file to analyze

        Returns:
            FileContextInfo with detected context and metadata
        """
        import asyncio
        try:
            # Try to get existing event loop
            loop = asyncio.get_running_loop()
            # If we're in an async context, we need a workaround
            import concurrent.futures
            with concurrent.futures.ThreadPoolExecutor() as pool:
                future = pool.submit(asyncio.run, self.analyze_file_async(file_path))
                return future.result()
        except RuntimeError:
            # No running loop, create one
            return asyncio.run(self.analyze_file_async(file_path))

    async def analyze_file_async(self, file_path: Path) -> FileContextInfo:
        """
        Analyze a single file to determine its context (async version).

        Args:
            file_path: Path to the file to analyze

        Returns:
            FileContextInfo with detected context and metadata
        """
        file_path = Path(file_path)

        # Multi-layer detection
        context, confidence, method = self._detect_context(file_path)

        # Use LLM if available and confidence is low
        if self.llm_analyzer and confidence < 0.7:
            try:
                # Read file content for LLM
                file_content = None
                try:
                    with open(file_path, 'r', encoding='utf-8', errors='ignore') as f:
                        file_content = f.read()[:5000]
                except:
                    pass

                # Enhance with LLM
                from warden.analysis.application.llm_context_analyzer import LlmContextAnalyzer
                if isinstance(self.llm_analyzer, LlmContextAnalyzer):
                    context, confidence, method = await self.llm_analyzer.analyze_file_context(
                        file_path=file_path,
                        initial_context=context,
                        initial_confidence=confidence,
                        file_content=file_content,
                    )
            except Exception as e:
                logger.debug(
                    "llm_enhancement_failed",
                    file=str(file_path),
                    error=str(e),
                )

        # Get appropriate weights for context
        weights = ContextWeights(context=context)

        # Determine suppression rules
        suppressed_issues = self._get_suppressed_issues(context)
        suppression_reason = self._get_suppression_reason(context)

        # Check for special markers
        has_ignore_marker = self._check_ignore_marker(file_path)
        is_entry_point = self._is_entry_point(file_path)
        is_generated = self._is_generated_file(file_path)
        is_vendor = self._is_vendor_file(file_path)

        return FileContextInfo(
            file_path=str(file_path),
            context=context,
            confidence=confidence,
            detection_method=method,
            weights=weights,
            suppressed_issues=suppressed_issues,
            suppression_reason=suppression_reason,
            is_entry_point=is_entry_point,
            is_generated=is_generated,
            is_vendor=is_vendor,
            has_ignore_marker=has_ignore_marker,
        )

    def _detect_context(self, file_path: Path) -> Tuple[FileContext, float, str]:
        """
        Detect file context using multiple strategies.

        Returns:
            (context, confidence, detection_method)
        """
        # Layer 1: Path-based detection (highest priority)
        context = self._detect_by_path(file_path)
        if context and context[1] > 0.9:
            return context[0], context[1], "path"

        # Layer 2: Content-based detection
        context = self._detect_by_content(file_path)
        if context and context[1] > 0.8:
            return context[0], context[1], "content"

        # Layer 3: Import analysis
        context = self._detect_by_imports(file_path)
        if context and context[1] > 0.7:
            return context[0], context[1], "imports"

        # Layer 4: Metadata/comments
        context = self._detect_by_metadata(file_path)
        if context:
            return context[0], context[1], "metadata"

        # Layer 5: Project context hints
        if self.project_context:
            context = self._detect_by_project_context(file_path)
            if context:
                return context[0], context[1], "project_context"

        # Default to production (safest assumption)
        return FileContext.PRODUCTION, 0.5, "default"

    def _compile_path_patterns(self) -> Dict[FileContext, List[re.Pattern]]:
        """Compile regex patterns for path-based detection."""
        patterns = {
            FileContext.TEST: [
                r"test[s]?/",
                r".*test_.*\.py$",
                r".*_test\.py$",
                r"spec[s]?/",
                r".*\.spec\.",
                r"__tests__/",
                r".*\.test\.",
            ],
            FileContext.EXAMPLE: [
                r"example[s]?/",
                r"demo[s]?/",
                r"sample[s]?/",
                r"tutorial[s]?/",
                r".*_example\.",
                r".*_demo\.",
            ],
            FileContext.FRAMEWORK: [
                r"\.warden/frames/",
                r"warden/validation/frames/",
                r".*_frame\.py$",
                r"frames?/",
            ],
            FileContext.DOCUMENTATION: [
                r"\.md$",
                r"\.rst$",
                r"docs?/",
                r"README",
                r"CHANGELOG",
                r"CONTRIBUTING",
            ],
            FileContext.CONFIGURATION: [
                r"\.ya?ml$",
                r"\.json$",
                r"\.toml$",
                r"\.ini$",
                r"\.cfg$",
                r"\.conf$",
                r"config/",
            ],
            FileContext.MIGRATION: [
                r"migrations?/",
                r"migrate/",
                r"alembic/",
                r".*_migration\.py$",
                r"db/migrations?/",
            ],
            FileContext.FIXTURE: [
                r"fixtures?/",
                r"mocks?/",
                r"stubs?/",
                r"fakes?/",
                r".*_fixture\.",
                r".*_mock\.",
            ],
            FileContext.VENDOR: [
                r"vendor/",
                r"node_modules/",
                r"bower_components/",
                r"third_party/",
                r"external/",
                r"\.vendor/",
            ],
            FileContext.GENERATED: [
                r"generated/",
                r"gen/",
                r"build/",
                r"dist/",
                r"_build/",
                r"__pycache__/",
                r".*\.pyc$",
                r".*_pb2\.py$",
                r".*_pb2_grpc\.py$",
            ],
            FileContext.SCRIPT: [
                r"scripts?/",
                r"bin/",
                r"tools?/",
                r"utils?/",
                r".*\.sh$",
                r".*\.bash$",
            ],
        }

        # Compile patterns
        compiled = {}
        for context, pattern_list in patterns.items():
            compiled[context] = [re.compile(p, re.IGNORECASE) for p in pattern_list]

        return compiled

    def _compile_content_patterns(self) -> Dict[FileContext, List[re.Pattern]]:
        """Compile regex patterns for content-based detection."""
        patterns = {
            FileContext.TEST: [
                r"import\s+pytest",
                r"from\s+pytest",
                r"import\s+unittest",
                r"from\s+unittest",
                r"class\s+Test\w+",
                r"def\s+test_\w+",
                r"@pytest\.",
                r"@mock\.",
                r"assert\s+.*==",
                r"self\.assert",
                r"describe\(",
                r"it\(",
                r"expect\(",
            ],
            FileContext.EXAMPLE: [
                r"#\s*Example:",
                r"#\s*Demo:",
                r"#\s*Sample:",
                r"#\s*Usage:",
                r"#\s*Bad:",
                r"#\s*Good:",
                r"#\s*DO NOT USE IN PRODUCTION",
                r"#\s*For demonstration",
            ],
            FileContext.FRAMEWORK: [
                r"class\s+\w+Frame\(",
                r"ValidationFrame",
                r"def\s+detect_pattern",
                r"def\s+check_security",
                r"#\s*Pattern definition",
                r"#\s*Frame implementation",
            ],
            FileContext.MIGRATION: [
                r"def\s+upgrade\(",
                r"def\s+downgrade\(",
                r"class\s+Migration",
                r"schema\.create_table",
                r"schema\.drop_table",
                r"ALTER\s+TABLE",
            ],
        }

        # Compile patterns
        compiled = {}
        for context, pattern_list in patterns.items():
            compiled[context] = [re.compile(p, re.MULTILINE) for p in pattern_list]

        return compiled

    def _detect_by_path(self, file_path: Path) -> Optional[Tuple[FileContext, float]]:
        """Detect context by file path patterns."""
        path_str = str(file_path)

        for context, patterns in self.path_patterns.items():
            for pattern in patterns:
                if pattern.search(path_str):
                    return (context, 0.95)

        return None

    def _detect_by_content(self, file_path: Path) -> Optional[Tuple[FileContext, float]]:
        """Detect context by file content patterns."""
        try:
            # Read first 5000 characters for performance
            with open(file_path, 'r', encoding='utf-8', errors='ignore') as f:
                content = f.read(5000)

            match_counts = {}
            for context, patterns in self.content_patterns.items():
                matches = 0
                for pattern in patterns:
                    if pattern.search(content):
                        matches += 1

                if matches > 0:
                    # Confidence based on match count
                    confidence = min(0.5 + (matches * 0.1), 0.95)
                    match_counts[context] = confidence

            if match_counts:
                # Return highest confidence match
                best_match = max(match_counts.items(), key=lambda x: x[1])
                return best_match

        except Exception as e:
            logger.debug(
                "content_detection_failed",
                file=str(file_path),
                error=str(e),
            )

        return None

    def _detect_by_imports(self, file_path: Path) -> Optional[Tuple[FileContext, float]]:
        """Detect context by import statements."""
        if not file_path.suffix == '.py':
            return None

        try:
            with open(file_path, 'r', encoding='utf-8', errors='ignore') as f:
                content = f.read()

            # Test framework imports
            test_imports = [
                "pytest", "unittest", "nose", "mock",
                "fixtures", "testtools", "hypothesis",
            ]

            for imp in test_imports:
                if f"import {imp}" in content or f"from {imp}" in content:
                    return (FileContext.TEST, 0.85)

            # Framework imports (Warden specific)
            if "from warden.validation.frames" in content:
                return (FileContext.FRAMEWORK, 0.80)

        except Exception:
            pass

        return None

    def _detect_by_metadata(self, file_path: Path) -> Optional[Tuple[FileContext, float]]:
        """Detect context by file metadata or special comments."""
        try:
            with open(file_path, 'r', encoding='utf-8', errors='ignore') as f:
                # Check first 1000 characters for metadata
                content = f.read(1000)

            # Check for explicit context declaration
            if "# warden-context:" in content:
                match = re.search(r"#\s*warden-context:\s*(\w+)", content)
                if match:
                    context_str = match.group(1).lower()
                    for context in FileContext:
                        if context.value == context_str:
                            return (context, 1.0)  # Explicit declaration

            # Check for generated file markers
            if "# Generated by" in content or "# Auto-generated" in content:
                return (FileContext.GENERATED, 0.9)

            # Check for vendor markers
            if "# Third-party" in content or "# External library" in content:
                return (FileContext.VENDOR, 0.9)

        except Exception:
            pass

        return None

    def _detect_by_project_context(self, file_path: Path) -> Optional[Tuple[FileContext, float]]:
        """Use project context hints for detection."""
        if not self.project_context:
            return None

        # Check against special directories
        path_str = str(file_path)

        if "test" in self.project_context.special_dirs:
            for test_dir in self.project_context.special_dirs["test"]:
                if test_dir in path_str:
                    return (FileContext.TEST, 0.8)

        if "vendor" in self.project_context.special_dirs:
            for vendor_dir in self.project_context.special_dirs["vendor"]:
                if vendor_dir in path_str:
                    return (FileContext.VENDOR, 0.9)

        return None

    def _get_suppressed_issues(self, context: FileContext) -> List[str]:
        """Get list of issue types to suppress for context."""
        suppression_map = {
            FileContext.TEST: [
                "sql_injection",
                "hardcoded_password",
                "hardcoded_secret",
                "weak_password",
                "insecure_random",
                "command_injection",
            ],
            FileContext.EXAMPLE: [
                "sql_injection",
                "xss",
                "hardcoded_password",
                "missing_error_handling",
                "insecure_config",
                "path_traversal",
            ],
            FileContext.FRAMEWORK: [
                "sql_injection",
                "xss",
                "command_injection",
                "path_traversal",
                "code_injection",
            ],
            FileContext.FIXTURE: [
                "hardcoded_password",
                "hardcoded_secret",
                "weak_password",
                "sensitive_data",
                "insecure_random",
            ],
            FileContext.DOCUMENTATION: ["*"],  # Suppress all
            FileContext.VENDOR: ["*"],  # Suppress all
            FileContext.GENERATED: ["*"],  # Suppress all
        }

        return suppression_map.get(context, [])

    def _get_suppression_reason(self, context: FileContext) -> Optional[str]:
        """Get suppression reason for context."""
        reasons = {
            FileContext.TEST: "Test files often contain intentional vulnerabilities",
            FileContext.EXAMPLE: "Example code demonstrates patterns including anti-patterns",
            FileContext.FRAMEWORK: "Framework code contains pattern definitions",
            FileContext.FIXTURE: "Test fixtures contain mock data",
            FileContext.DOCUMENTATION: "Documentation contains code examples",
            FileContext.VENDOR: "Third-party code should not be modified",
            FileContext.GENERATED: "Generated code should not be manually modified",
        }

        return reasons.get(context)

    def _check_ignore_marker(self, file_path: Path) -> bool:
        """Check if file has ignore marker."""
        try:
            with open(file_path, 'r', encoding='utf-8', errors='ignore') as f:
                content = f.read(1000)

            ignore_markers = [
                "warden-ignore",
                "warden:ignore",
                "noqa: warden",
                "pylint: skip-file",
            ]

            return any(marker in content for marker in ignore_markers)

        except Exception:
            return False

    def _is_entry_point(self, file_path: Path) -> bool:
        """Check if file is an entry point."""
        entry_names = [
            "main.py", "app.py", "index.py", "__main__.py",
            "run.py", "start.py", "server.py", "wsgi.py",
            "index.js", "app.js", "main.js", "server.js",
        ]

        return file_path.name in entry_names

    def _is_generated_file(self, file_path: Path) -> bool:
        """Check if file is generated."""
        # Check path
        path_str = str(file_path)
        generated_patterns = [
            "__pycache__", ".pyc", "_pb2.py", "_pb2_grpc.py",
            "generated/", "gen/", "build/", "dist/",
        ]

        return any(pattern in path_str for pattern in generated_patterns)

    def _is_vendor_file(self, file_path: Path) -> bool:
        """Check if file is vendor/third-party."""
        path_str = str(file_path)
        vendor_patterns = [
            "vendor/", "node_modules/", "bower_components/",
            "third_party/", "external/", ".vendor/",
        ]

        return any(pattern in path_str for pattern in vendor_patterns)


"""
Tests for PipelineOrchestrator.

Validates sequential, parallel, and fail-fast execution strategies.
"""

import pytest
from warden.pipeline import (
    PipelineOrchestrator,
    PipelineConfig,
    ExecutionStrategy,
    PipelineStatus,
)
from warden.validation.frames import SecurityFrame, ChaosFrame
from warden.validation.domain.frame import CodeFile


@pytest.mark.asyncio
async def test_orchestrator_sequential_execution():
    """Test sequential execution of frames."""
    frames = [SecurityFrame(), ChaosFrame()]
    config = PipelineConfig(strategy=ExecutionStrategy.SEQUENTIAL, fail_fast=False)

    orchestrator = PipelineOrchestrator(frames=frames, config=config)

    code_file = CodeFile(
        path="test.py",
        content='password = "admin123"',  # Security issue
        language="python",
    )

    result, _ = await orchestrator.execute([code_file], frames_to_execute=["security", "chaos"])

    # Both frames should execute
    assert result.total_frames == 2
    assert result.frames_passed + result.frames_failed == 2
    assert result.status == PipelineStatus.FAILED  # Security blocker failed


@pytest.mark.asyncio
async def test_orchestrator_parallel_execution():
    """Test parallel execution of frames."""
    frames = [SecurityFrame(), ChaosFrame()]
    config = PipelineConfig(
        strategy=ExecutionStrategy.PARALLEL,
        parallel_limit=2,
        fail_fast=False,
    )

    orchestrator = PipelineOrchestrator(frames=frames, config=config)

    code_file = CodeFile(
        path="test.py",
        content='OPENAI_API_KEY = "sk-1234567890abcdefghijklmnopqrstuvwxyz123456789012"',  # Security issue - full OpenAI key
        language="python",
    )

    result, _ = await orchestrator.execute([code_file], frames_to_execute=["security", "chaos"])

    # Both frames should execute in parallel
    assert result.total_frames == 2
    assert result.total_findings > 0


@pytest.mark.asyncio
async def test_orchestrator_fail_fast():
    """Test fail-fast execution stops on blocker failure."""
    frames = [SecurityFrame(), ChaosFrame()]
    config = PipelineConfig(strategy=ExecutionStrategy.FAIL_FAST)

    orchestrator = PipelineOrchestrator(frames=frames, config=config)

    code_file = CodeFile(
        path="test.py",
        content='query = f"SELECT * FROM users WHERE id = {user_id}"',  # SQL injection
        language="python",
    )

    result, _ = await orchestrator.execute([code_file], frames_to_execute=["security", "chaos"])

    # Should stop after SecurityFrame fails (it's a blocker)
    assert result.status == PipelineStatus.FAILED
    assert result.frames_failed >= 1
    # ChaosFrame may be skipped if SecurityFrame is blocker and failed
    assert result.frames_skipped >= 0


@pytest.mark.asyncio
async def test_orchestrator_passes_clean_code():
    """Test orchestrator passes clean code."""
    frames = [SecurityFrame(), ChaosFrame()]
    config = PipelineConfig(strategy=ExecutionStrategy.SEQUENTIAL, fail_fast=False)

    orchestrator = PipelineOrchestrator(frames=frames, config=config)

    code_file = CodeFile(
        path="test.py",
        content="""
# Clean code with no security or chaos issues
def add_numbers(a: int, b: int) -> int:
    '''Add two numbers together.'''
    return a + b

def multiply_numbers(a: int, b: int) -> int:
    '''Multiply two numbers together.'''
    return a * b
""",
        language="python",
    )

    result, _ = await orchestrator.execute([code_file], frames_to_execute=["security", "chaos"])

    # Should pass all frames
    assert result.status == PipelineStatus.COMPLETED
    assert result.passed is True
    assert result.total_findings == 0


@pytest.mark.asyncio
async def test_orchestrator_frame_priority_sorting():
    """Test frames are sorted by priority."""
    # ChaosFrame has priority HIGH, SecurityFrame has CRITICAL
    frames = [ChaosFrame(), SecurityFrame()]  # Wrong order
    config = PipelineConfig(strategy=ExecutionStrategy.SEQUENTIAL, fail_fast=False)

    orchestrator = PipelineOrchestrator(frames=frames, config=config)

    # After initialization, frames should be sorted by priority (lower value = higher priority)
    assert orchestrator.frames[0].priority.value == 1  # CRITICAL (Security)
    assert orchestrator.frames[1].priority.value == 2  # HIGH (Chaos)


@pytest.mark.asyncio
async def test_orchestrator_multiple_files():
    """Test orchestrator handles multiple files."""
    frames = [SecurityFrame()]
    config = PipelineConfig(strategy=ExecutionStrategy.SEQUENTIAL, fail_fast=False)

    orchestrator = PipelineOrchestrator(frames=frames, config=config)

    code_files = [
        CodeFile(path="file1.py", content='pwd = "123"', language="python"),
        CodeFile(path="file2.py", content='token = "abc"', language="python"),
        CodeFile(
            path="file3.py",
            content="# Clean file\nimport os",
            language="python",
        ),
    ]

    result, _ = await orchestrator.execute(code_files)

    # Should process all 3 files
    assert result.total_findings >= 2  # At least 2 issues from file1 and file2


@pytest.mark.asyncio
async def test_orchestrator_result_structure():
    """Test pipeline result has correct Panel JSON structure."""
    frames = [SecurityFrame()]
    config = PipelineConfig(strategy=ExecutionStrategy.SEQUENTIAL, fail_fast=False)

    orchestrator = PipelineOrchestrator(frames=frames, config=config)

    code_file = CodeFile(
        path="test.py",
        content='password = "admin"',
        language="python",
    )

    result, _ = await orchestrator.execute([code_file])

    # Test Panel JSON compatibility
    json_data = result.to_json()

    # Check camelCase fields
    assert "pipelineId" in json_data
    assert "pipelineName" in json_data
    assert "status" in json_data
    assert "duration" in json_data
    assert "totalFrames" in json_data
    assert "framesPassed" in json_data
    assert "framesFailed" in json_data
    assert "totalFindings" in json_data
    assert "frameResults" in json_data

    # Status should be integer
    assert isinstance(json_data["status"], int)


@pytest.mark.asyncio
async def test_orchestrator_severity_counts():
    """Test orchestrator correctly counts findings by severity."""
    frames = [SecurityFrame(), ChaosFrame()]
    config = PipelineConfig(strategy=ExecutionStrategy.SEQUENTIAL, fail_fast=False)

    orchestrator = PipelineOrchestrator(frames=frames, config=config)

    code_file = CodeFile(
        path="test.py",
        content="""
# Multiple issues with different severities
password = "admin123"  # CRITICAL (hardcoded password)
query = f"SELECT * FROM users WHERE id = {user_id}"  # CRITICAL (SQL injection)
import requests
response = requests.get(url)  # HIGH (missing timeout)
""",
        language="python",
    )

    result, _ = await orchestrator.execute([code_file], frames_to_execute=["security", "chaos"])

    # Should have findings across multiple severity levels
    assert result.total_findings > 0
    assert result.critical_findings > 0 or result.high_findings > 0


@pytest.mark.asyncio
async def test_orchestrator_metadata():
    """Test pipeline result includes execution metadata."""
    frames = [SecurityFrame()]
    config = PipelineConfig(
        strategy=ExecutionStrategy.SEQUENTIAL,
        fail_fast=True,
    )

    orchestrator = PipelineOrchestrator(frames=frames, config=config)

    code_file = CodeFile(
        path="test.py",
        content='api_key = "sk-123"',
        language="python",
    )

    result, _ = await orchestrator.execute([code_file])

    # Check metadata
    assert "strategy" in result.metadata
    assert result.metadata["strategy"] == "sequential"
    assert "fail_fast" in result.metadata
    assert result.metadata["fail_fast"] is True
    assert "frame_executions" in result.metadata


@pytest.mark.asyncio
async def test_orchestrator_has_blockers_property():
    """Test has_blockers property works correctly."""
    frames = [SecurityFrame()]  # Blocker frame
    config = PipelineConfig(strategy=ExecutionStrategy.SEQUENTIAL, fail_fast=False)

    orchestrator = PipelineOrchestrator(frames=frames, config=config)

    # Code with security issue
    code_file = CodeFile(
        path="test.py",
        content='password = "admin"',
        language="python",
    )

    result, _ = await orchestrator.execute([code_file])

    # Should have blocker issues
    assert result.has_blockers is True


@pytest.mark.asyncio
async def test_orchestrator_no_blockers():
    """Test has_blockers is False when only warnings."""
    frames = [ChaosFrame()]  # Non-blocker frame
    config = PipelineConfig(strategy=ExecutionStrategy.SEQUENTIAL, fail_fast=False)

    orchestrator = PipelineOrchestrator(frames=frames, config=config)

    # Code with chaos issues (warnings only)
    code_file = CodeFile(
        path="test.py",
        content="""
import requests
response = requests.get(url)  # Missing timeout (warning)
""",
        language="python",
    )

    result, _ = await orchestrator.execute([code_file])

    # Should not have blocker issues (ChaosFrame is not a blocker)
    assert result.has_blockers is False



"""
Resilience Architecture Analysis Frame (formerly Chaos Frame).

Validates architectural resilience using LLM-based Failure Mode & Effects Analysis (FMEA).
"""

import time
from typing import List, Dict, Any

from warden.validation.domain.frame import (
    ValidationFrame,
    FrameResult,
    Finding,
    CodeFile,
)
from warden.validation.domain.enums import (
    FrameCategory,
    FramePriority,
    FrameScope,
    FrameApplicability,
)
from warden.shared.infrastructure.logging import get_logger
from warden.llm.providers.base import ILlmClient

logger = get_logger(__name__)


class ResilienceFrame(ValidationFrame):
    """
    Validation frame for Resilience Architecture Analysis (Chaos 2.0).
    
    This frame uses LLMs to perform Failure Mode & Effects Analysis (FMEA),
    identifying architectural weaknesses, critical paths, state consistency issues,
    and graceful degradation flaws.
    """
    
    # Metadata
    name = "Resilience Architecture Analysis"
    description = "LLM-driven Failure Mode & Effects Analysis (FMEA) for architectural resilience."
    category = FrameCategory.GLOBAL
    priority = FramePriority.HIGH
    scope = FrameScope.FILE_LEVEL
    is_blocker = False  # Not blocking for now as it's advisory
    version = "2.0.0"
    author = "Warden Team"
    applicability = [FrameApplicability.ALL]

    def __init__(self, config: Dict[str, Any] | None = None) -> None:
        """Initialize Resilience Frame."""
        super().__init__(config)
        
        # Load System Prompt
        try:
            from warden.llm.prompts.resilience import CHAOS_SYSTEM_PROMPT
            self.system_prompt = CHAOS_SYSTEM_PROMPT
        except ImportError:
            logger.warning("resilience_prompt_import_failed")
            self.system_prompt = "You are a Resilience Engineer."

    async def execute(self, code_file: CodeFile) -> FrameResult:
        """
        Execute resilience analysis on code file.

        Args:
            code_file: Code file to validate

        Returns:
            FrameResult with findings
        """
        start_time = time.perf_counter()

        logger.info(
            "resilience_analysis_started",
            file_path=code_file.path,
            language=code_file.language,
            has_llm_service=hasattr(self, 'llm_service'),
            llm_service_type=str(type(getattr(self, 'llm_service', None))) if hasattr(self, 'llm_service') else "N/A"
        )

        findings: List[Finding] = []

        # Run LLM analysis if available (PRIMARY METHOD)
        if hasattr(self, 'llm_service') and self.llm_service:
            llm_findings = await self._analyze_with_llm(code_file)
            findings.extend(llm_findings)
        else:
            logger.warning("resilience_llm_not_available_skipping_analysis")

        # Determine status
        status = self._determine_status(findings)

        duration = time.perf_counter() - start_time

        logger.info(
            "resilience_analysis_completed",
            file_path=code_file.path,
            status=status,
            total_findings=len(findings),
            duration=f"{duration:.2f}s",
        )

        return FrameResult(
            frame_id=self.frame_id,
            frame_name=self.name,
            status=status,
            duration=duration,
            issues_found=len(findings),
            is_blocker=self.is_blocker,
            findings=findings,
            metadata={
                "method": "llm_fmea",
                "file_size": code_file.size_bytes,
                "line_count": code_file.line_count,
            },
        )

    async def _analyze_with_llm(self, code_file: CodeFile) -> List[Finding]:
        """
        Analyze code using LLM for Resilience FMEA.
        """
        from warden.llm.prompts.resilience import CHAOS_SYSTEM_PROMPT, generate_chaos_request
        from warden.llm.types import LlmRequest, AnalysisResult
        
        findings: List[Finding] = []
        try:
            logger.info("resilience_llm_analysis_started", file=code_file.path)
            
            client: ILlmClient = self.llm_service
            
            request = LlmRequest(
                system_prompt=CHAOS_SYSTEM_PROMPT,
                user_message=generate_chaos_request(code_file.content, code_file.language, code_file.path),
                temperature=0.2  # Slightly higher for creative scenario generation
            )
            
            response = await client.send_async(request)
            
            if response.success and response.content:
                # Use robust shared JSON parser
                from warden.shared.utils.json_parser import parse_json_from_llm
                json_data = parse_json_from_llm(response.content)
                
                if json_data:
                    try:
                        # Parse result with Pydantic
                        result = AnalysisResult.from_json(json_data)
                        
                        for issue in result.issues:
                            findings.append(Finding(
                                id=f"{self.frame_id}-resilience-{issue.line}",
                                severity=issue.severity,
                                message=issue.title,
                                location=f"{code_file.path}:{issue.line}",
                                detail=f"{issue.description}\n\nSuggestion: {issue.suggestion}",
                                code=issue.evidence_quote
                            ))
                        
                        logger.info("resilience_llm_analysis_completed", 
                                  findings=len(findings), 
                                  confidence=result.confidence,
                                  resilience_score=result.score)
                                  
                    except (ValueError, TypeError, KeyError) as e:
                        logger.warning("resilience_llm_parsing_failed", error=str(e), content_preview=response.content[:100])
                else:
                    logger.warning("resilience_llm_response_not_json", content_preview=response.content[:100])
            else:
                 logger.warning("resilience_llm_request_failed", error=response.error_message)

        except (RuntimeError, AttributeError, ValueError) as e:
            logger.error("resilience_llm_error", error=str(e))
            
        return findings

    def _determine_status(self, findings: List[Finding]) -> str:
        """Determine frame status based on findings."""
        if not findings:
            return "passed"

        critical_count = sum(1 for f in findings if f.severity == "critical")
        high_count = sum(1 for f in findings if f.severity == "high")

        if critical_count > 0:
            return "failed"
        elif high_count > 0:
            return "warning"
        
        return "passed"



"""
Base LLM Client Interface

Based on C# ILlmClient.cs:
/Users/alper/vibe-code-analyzer/src/Warden.LLM/ILlmClient.cs

All provider implementations must inherit from this interface
"""

from abc import ABC, abstractmethod
import json
from ..types import LlmProvider, LlmRequest, LlmResponse


class ILlmClient(ABC):
    """
    Interface for LLM providers

    Matches C# ILlmClient interface
    All providers (Anthropic, DeepSeek, QwenCode, etc.) must implement this
    """

    @property
    @abstractmethod
    def provider(self) -> LlmProvider:
        """
        The provider type

        Returns:
            LlmProvider enum value
        """
        pass

    @abstractmethod
    async def send_async(self, request: LlmRequest) -> LlmResponse:
        """
        Send a request to the LLM provider

        Args:
            request: The LLM request parameters

        Returns:
            LLM response with content or error

        Raises:
            Should NOT raise exceptions - return LlmResponse with success=False instead
        """
        pass

    @abstractmethod
    async def is_available_async(self) -> bool:
        """
        Check if the provider is available/configured

        Returns:
            True if the provider is ready to use, False otherwise

        Note:
            Should NOT raise exceptions - return False on any error
        """
        pass

    async def complete_async(self, prompt: str, system_prompt: str = "You are a helpful coding assistant.") -> LlmResponse:
        """
        Simple completion method for non-streaming requests.

        Args:
            prompt: User prompt
            system_prompt: System prompt (optional)

        Returns:
            LlmResponse with content and token usage

        Raises:
            Exception: If request fails
        """
        # Default implementation using send_async
        request = LlmRequest(
            user_message=prompt,
            system_prompt=system_prompt,
            model=None,  # Use provider default
            temperature=0.7,
            max_tokens=2000,
            timeout_seconds=30.0
        )

        response = await self.send_async(request)

        if not response.success:
            raise Exception(f"LLM request failed: {response.error_message}")

        return response

    async def analyze_security_async(self, code_content: str, language: str) -> dict:
        """
        Analyze code for security vulnerabilities using LLM.
        
        Default implementation uses complete_async with a standard prompt.
        Providers may override this for specialized models or parameters.
        
        Args:
            code_content: Source code to analyze
            language: Language of the code
            
        Returns:
            Dict containing findings list
        """
        from warden.shared.utils.json_parser import parse_json_from_llm
        from typing import Dict, Any

        prompt = f"""
        You are a senior security researcher. Analyze this {language} code for critical vulnerabilities.
        Target vulnerabilities: SSRF, CSRF, XXE, Insecure Deserialization, Path Traversal, and Command Injection.
        
        Ignore stylistic issues. Focus only on exploitable security flaws.
        
        Return a JSON object in this exact format:
        {{
            "findings": [
                {{
                    "severity": "critical|high|medium",
                    "message": "Short description",
                    "line_number": 1,
                    "detail": "Detailed explanation of the exploit vector"
                }}
            ]
        }}
        
        If no issues found, return {{ "findings": [] }}.
        
        Code:
        ```{language}
        {code_content[:4000]}
        ```
        """
        
        try:
            response = await self.complete_async(prompt, system_prompt="You are a strict security auditor. Output valid JSON only.")
            if not response.success:
                return {"findings": []}
                
            parsed = parse_json_from_llm(response.content)
            return parsed or {"findings": []}
        except (json.JSONDecodeError, ValueError, KeyError) as e:
            # Log but don't crash - return safe default
            return {"findings": []}



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: warden.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'warden.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0cwarden.proto\x12\x06warden\"\x07\n\x05\x45mpty\"\xe2\x02\n\x07\x46inding\x12\n\n\x02id\x18\x01 \x01(\t\x12\r\n\x05title\x18\x02 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x03 \x01(\t\x12\"\n\x08severity\x18\x04 \x01(\x0e\x32\x10.warden.Severity\x12\x11\n\tfile_path\x18\x05 \x01(\t\x12\x13\n\x0bline_number\x18\x06 \x01(\x05\x12\x15\n\rcolumn_number\x18\x07 \x01(\x05\x12\x14\n\x0c\x63ode_snippet\x18\x08 \x01(\t\x12\x12\n\nsuggestion\x18\t \x01(\t\x12\x10\n\x08\x66rame_id\x18\n \x01(\t\x12\x0e\n\x06\x63we_id\x18\x0b \x01(\t\x12\x16\n\x0eowasp_category\x18\x0c \x01(\t\x12/\n\x08metadata\x18\r \x03(\x0b\x32\x1d.warden.Finding.MetadataEntry\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\xcd\x01\n\rFortification\x12\n\n\x02id\x18\x01 \x01(\t\x12\r\n\x05title\x18\x02 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x03 \x01(\t\x12\x11\n\tfile_path\x18\x04 \x01(\t\x12\x13\n\x0bline_number\x18\x05 \x01(\x05\x12\x15\n\roriginal_code\x18\x06 \x01(\t\x12\x16\n\x0esuggested_code\x18\x07 \x01(\t\x12\x11\n\trationale\x18\x08 \x01(\t\x12\"\n\x08priority\x18\t \x01(\x0e\x32\x10.warden.Severity\"r\n\x08\x43leaning\x12\n\n\x02id\x18\x01 \x01(\t\x12\r\n\x05title\x18\x02 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x03 \x01(\t\x12\x11\n\tfile_path\x18\x04 \x01(\t\x12\x13\n\x0bline_number\x18\x05 \x01(\x05\x12\x0e\n\x06\x64\x65tail\x18\x06 \x01(\t\"\xf5\x03\n\x05Issue\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0c\n\x04hash\x18\x02 \x01(\t\x12\r\n\x05title\x18\x03 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x04 \x01(\t\x12\"\n\x08severity\x18\x05 \x01(\x0e\x32\x10.warden.Severity\x12!\n\x05state\x18\x06 \x01(\x0e\x32\x12.warden.IssueState\x12\x11\n\tfile_path\x18\x07 \x01(\t\x12\x13\n\x0bline_number\x18\x08 \x01(\x05\x12\x14\n\x0c\x63ode_snippet\x18\t \x01(\t\x12\x10\n\x08\x66rame_id\x18\n \x01(\t\x12\x16\n\x0e\x66irst_detected\x18\x0b \x01(\t\x12\x11\n\tlast_seen\x18\x0c \x01(\t\x12\x13\n\x0bresolved_at\x18\r \x01(\t\x12\x13\n\x0bresolved_by\x18\x0e \x01(\t\x12\x15\n\rsuppressed_at\x18\x0f \x01(\t\x12\x15\n\rsuppressed_by\x18\x10 \x01(\t\x12\x1a\n\x12suppression_reason\x18\x11 \x01(\t\x12\x18\n\x10occurrence_count\x18\x12 \x01(\x05\x12-\n\x08metadata\x18\x13 \x03(\x0b\x32\x1b.warden.Issue.MetadataEntry\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\xa4\x01\n\x0bIssueFilter\x12\"\n\x06states\x18\x01 \x03(\x0e\x32\x12.warden.IssueState\x12$\n\nseverities\x18\x02 \x03(\x0e\x32\x10.warden.Severity\x12\x11\n\tframe_ids\x18\x03 \x03(\t\x12\x19\n\x11\x66ile_path_pattern\x18\x04 \x01(\t\x12\r\n\x05limit\x18\x05 \x01(\x05\x12\x0e\n\x06offset\x18\x06 \x01(\x05\"W\n\tIssueList\x12\x1d\n\x06issues\x18\x01 \x03(\x0b\x32\r.warden.Issue\x12\x13\n\x0btotal_count\x18\x02 \x01(\x05\x12\x16\n\x0e\x66iltered_count\x18\x03 \x01(\x05\" \n\x0cIssueRequest\x12\x10\n\x08issue_id\x18\x01 \x01(\t\" \n\x10IssueHashRequest\x12\x0c\n\x04hash\x18\x01 \x01(\t\"F\n\x12IssueActionRequest\x12\x10\n\x08issue_id\x18\x01 \x01(\t\x12\r\n\x05\x61\x63tor\x18\x02 \x01(\t\x12\x0f\n\x07\x63omment\x18\x03 \x01(\t\"[\n\x13IssueActionResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x1c\n\x05issue\x18\x02 \x01(\x0b\x32\r.warden.Issue\x12\x15\n\rerror_message\x18\x03 \x01(\t\"\xdd\x02\n\nIssueStats\x12\r\n\x05total\x18\x01 \x01(\x05\x12\x0c\n\x04open\x18\x02 \x01(\x05\x12\x10\n\x08resolved\x18\x03 \x01(\x05\x12\x12\n\nsuppressed\x18\x04 \x01(\x05\x12\x10\n\x08\x63ritical\x18\x05 \x01(\x05\x12\x0c\n\x04high\x18\x06 \x01(\x05\x12\x0e\n\x06medium\x18\x07 \x01(\x05\x12\x0b\n\x03low\x18\x08 \x01(\x05\x12\x0c\n\x04info\x18\t \x01(\x05\x12\x31\n\x08\x62y_frame\x18\n \x03(\x0b\x32\x1f.warden.IssueStats.ByFrameEntry\x12/\n\x07\x62y_file\x18\x0b \x03(\x0b\x32\x1e.warden.IssueStats.ByFileEntry\x1a.\n\x0c\x42yFrameEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x05:\x02\x38\x01\x1a-\n\x0b\x42yFileEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x05:\x02\x38\x01\"8\n\x0cIssueHistory\x12(\n\tsnapshots\x18\x01 \x03(\x0b\x32\x15.warden.IssueSnapshot\"\xc2\x01\n\rIssueSnapshot\x12\x13\n\x0bsnapshot_id\x18\x01 \x01(\t\x12\x11\n\ttimestamp\x18\x02 \x01(\t\x12\x0e\n\x06run_id\x18\x03 \x01(\t\x12\x14\n\x0ctotal_issues\x18\x04 \x01(\x05\x12\x12\n\nnew_issues\x18\x05 \x01(\x05\x12\x17\n\x0fresolved_issues\x18\x06 \x01(\x05\x12\x17\n\x0freopened_issues\x18\x07 \x01(\x05\x12\x1d\n\x06issues\x18\x08 \x03(\x0b\x32\r.warden.Issue\"x\n\x15\x41nalyzeResultsRequest\x12&\n\x06result\x18\x01 \x01(\x0b\x32\x16.warden.PipelineResult\x12\x12\n\nproject_id\x18\x02 \x01(\t\x12\x0e\n\x06\x62ranch\x18\x03 \x01(\t\x12\x13\n\x0b\x63ommit_hash\x18\x04 \x01(\t\"\xe4\x02\n\x0e\x41nalysisResult\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12%\n\x05trend\x18\x02 \x01(\x0e\x32\x16.warden.TrendDirection\x12\x15\n\rquality_score\x18\x03 \x01(\x02\x12\x12\n\nnew_issues\x18\x04 \x01(\x05\x12\x17\n\x0fresolved_issues\x18\x05 \x01(\x05\x12\x17\n\x0freopened_issues\x18\x06 \x01(\x05\x12\x19\n\x11persistent_issues\x18\x07 \x01(\x05\x12.\n\x0ftrending_issues\x18\x08 \x03(\x0b\x32\x15.warden.TrendingIssue\x12\'\n\x0b\x66rame_stats\x18\t \x01(\x0b\x32\x12.warden.FrameStats\x12-\n\x0eseverity_stats\x18\n \x01(\x0b\x32\x15.warden.SeverityStats\x12\x1a\n\x12\x61nalysis_timestamp\x18\x0b \x01(\t\"s\n\rTrendingIssue\x12\x12\n\nissue_hash\x18\x01 \x01(\t\x12\r\n\x05title\x18\x02 \x01(\t\x12\x18\n\x10occurrence_count\x18\x03 \x01(\x05\x12%\n\x05trend\x18\x04 \x01(\x0e\x32\x16.warden.TrendDirection\"{\n\nFrameStats\x12,\n\x05stats\x18\x01 \x03(\x0b\x32\x1d.warden.FrameStats.StatsEntry\x1a?\n\nStatsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12 \n\x05value\x18\x02 \x01(\x0b\x32\x11.warden.FrameStat:\x02\x38\x01\"\xb7\x01\n\tFrameStat\x12\x10\n\x08\x66rame_id\x18\x01 \x01(\t\x12\x12\n\nframe_name\x18\x02 \x01(\t\x12\x16\n\x0etotal_findings\x18\x03 \x01(\x05\x12\x10\n\x08\x63ritical\x18\x04 \x01(\x05\x12\x0c\n\x04high\x18\x05 \x01(\x05\x12\x0e\n\x06medium\x18\x06 \x01(\x05\x12\x0b\n\x03low\x18\x07 \x01(\x05\x12\x14\n\x0c\x61vg_severity\x18\x08 \x01(\x02\x12\x19\n\x11\x65xecution_time_ms\x18\t \x01(\x03\"r\n\rSeverityStats\x12\x10\n\x08\x63ritical\x18\x01 \x01(\x05\x12\x0c\n\x04high\x18\x02 \x01(\x05\x12\x0e\n\x06medium\x18\x03 \x01(\x05\x12\x0b\n\x03low\x18\x04 \x01(\x05\x12\x0c\n\x04info\x18\x05 \x01(\x05\x12\x16\n\x0eweighted_score\x18\x06 \x01(\x02\"\x1d\n\x0cTrendRequest\x12\r\n\x05limit\x18\x01 \x01(\x05\"b\n\rTrendResponse\x12-\n\roverall_trend\x18\x01 \x01(\x0e\x32\x16.warden.TrendDirection\x12\"\n\x06points\x18\x02 \x03(\x0b\x32\x12.warden.TrendPoint\"L\n\nTrendPoint\x12\x11\n\ttimestamp\x18\x01 \x01(\t\x12\x14\n\x0ctotal_issues\x18\x02 \x01(\x05\x12\x15\n\rquality_score\x18\x03 \x01(\x02\"\xa6\x01\n\x14QualityScoreResponse\x12\r\n\x05score\x18\x01 \x01(\x02\x12\r\n\x05grade\x18\x02 \x01(\t\x12>\n\tbreakdown\x18\x03 \x03(\x0b\x32+.warden.QualityScoreResponse.BreakdownEntry\x1a\x30\n\x0e\x42reakdownEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x02:\x02\x38\x01\"\xd7\x01\n\rReportRequest\x12\x0e\n\x06run_id\x18\x01 \x01(\t\x12\x13\n\x0boutput_path\x18\x02 \x01(\t\x12\x14\n\x0cproject_name\x18\x03 \x01(\t\x12\x0e\n\x06\x62ranch\x18\x04 \x01(\t\x12\x13\n\x0b\x63ommit_hash\x18\x05 \x01(\t\x12\x35\n\x08metadata\x18\x06 \x03(\x0b\x32#.warden.ReportRequest.MetadataEntry\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\x83\x01\n\x0eReportResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x11\n\treport_id\x18\x02 \x01(\t\x12\x11\n\tfile_path\x18\x03 \x01(\t\x12\x0f\n\x07\x63ontent\x18\x04 \x01(\t\x12\x12\n\nsize_bytes\x18\x05 \x01(\x03\x12\x15\n\rerror_message\x18\x06 \x01(\t\"(\n\x13ReportStatusRequest\x12\x11\n\treport_id\x18\x01 \x01(\t\"u\n\x14ReportStatusResponse\x12\x11\n\treport_id\x18\x01 \x01(\t\x12\x0e\n\x06status\x18\x02 \x01(\t\x12\x10\n\x08progress\x18\x03 \x01(\x02\x12\x11\n\tfile_path\x18\x04 \x01(\t\x12\x15\n\rerror_message\x18\x05 \x01(\t\"g\n\rSearchRequest\x12\r\n\x05query\x18\x01 \x01(\t\x12\x10\n\x08language\x18\x02 \x01(\t\x12\r\n\x05limit\x18\x03 \x01(\x05\x12\x11\n\tmin_score\x18\x04 \x01(\x02\x12\x13\n\x0b\x63hunk_types\x18\x05 \x03(\t\"a\n\x0cSearchResult\x12\"\n\x07results\x18\x01 \x03(\x0b\x32\x11.warden.CodeChunk\x12\x15\n\rtotal_matches\x18\x02 \x01(\x05\x12\x16\n\x0esearch_time_ms\x18\x03 \x01(\x03\"\x93\x02\n\tCodeChunk\x12\n\n\x02id\x18\x01 \x01(\t\x12\x11\n\tfile_path\x18\x02 \x01(\t\x12\x12\n\nchunk_type\x18\x03 \x01(\t\x12\x0c\n\x04name\x18\x04 \x01(\t\x12\x0f\n\x07\x63ontent\x18\x05 \x01(\t\x12\x12\n\nstart_line\x18\x06 \x01(\x05\x12\x10\n\x08\x65nd_line\x18\x07 \x01(\x05\x12\x10\n\x08language\x18\x08 \x01(\t\x12\x18\n\x10similarity_score\x18\t \x01(\x02\x12\x31\n\x08metadata\x18\n \x03(\x0b\x32\x1f.warden.CodeChunk.MetadataEntry\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"V\n\x12SimilarCodeRequest\x12\x0c\n\x04\x63ode\x18\x01 \x01(\t\x12\x10\n\x08language\x18\x02 \x01(\t\x12\r\n\x05limit\x18\x03 \x01(\x05\x12\x11\n\tmin_score\x18\x04 \x01(\x02\"F\n\x0cIndexRequest\x12\x0c\n\x04path\x18\x01 \x01(\t\x12\x15\n\rforce_reindex\x18\x02 \x01(\x08\x12\x11\n\tlanguages\x18\x03 \x03(\t\"{\n\rIndexResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x15\n\rfiles_indexed\x18\x02 \x01(\x05\x12\x16\n\x0e\x63hunks_created\x18\x03 \x01(\x05\x12\x13\n\x0b\x64uration_ms\x18\x04 \x01(\x03\x12\x15\n\rerror_message\x18\x05 \x01(\t\"\xd9\x02\n\nIndexStats\x12\x13\n\x0btotal_files\x18\x01 \x01(\x05\x12\x14\n\x0ctotal_chunks\x18\x02 \x01(\x05\x12\x44\n\x12\x63hunks_by_language\x18\x03 \x03(\x0b\x32(.warden.IndexStats.ChunksByLanguageEntry\x12<\n\x0e\x63hunks_by_type\x18\x04 \x03(\x0b\x32$.warden.IndexStats.ChunksByTypeEntry\x12\x14\n\x0clast_indexed\x18\x05 \x01(\t\x12\x18\n\x10index_size_bytes\x18\x06 \x01(\x03\x1a\x37\n\x15\x43hunksByLanguageEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x05:\x02\x38\x01\x1a\x33\n\x11\x43hunksByTypeEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x05:\x02\x38\x01\"}\n\x0f\x44iscoverRequest\x12\x0c\n\x04path\x18\x01 \x01(\t\x12\x11\n\tmax_depth\x18\x02 \x01(\x05\x12\x15\n\ruse_gitignore\x18\x03 \x01(\x08\x12\x18\n\x10include_patterns\x18\x04 \x03(\t\x12\x18\n\x10\x65xclude_patterns\x18\x05 \x03(\t\"\xe4\x01\n\x10\x44iscoverResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x13\n\x0btotal_files\x18\x02 \x01(\x05\x12\x18\n\x10\x61nalyzable_files\x18\x03 \x01(\x05\x12%\n\x05\x66iles\x18\x04 \x03(\x0b\x32\x16.warden.DiscoveredFile\x12-\n\nframeworks\x18\x05 \x03(\x0b\x32\x19.warden.DetectedFramework\x12#\n\x05stats\x18\x06 \x01(\x0b\x32\x14.warden.ProjectStats\x12\x15\n\rerror_message\x18\x07 \x01(\t\"\x94\x01\n\x0e\x44iscoveredFile\x12\x0c\n\x04path\x18\x01 \x01(\t\x12#\n\tfile_type\x18\x02 \x01(\x0e\x32\x10.warden.FileType\x12\x12\n\nsize_bytes\x18\x03 \x01(\x03\x12\x12\n\nline_count\x18\x04 \x01(\x05\x12\x15\n\ris_analyzable\x18\x05 \x01(\x08\x12\x10\n\x08language\x18\x06 \x01(\t\"1\n\x0e\x46ileTypeFilter\x12\x1f\n\x05types\x18\x01 \x03(\x0e\x32\x10.warden.FileType\"o\n\x11\x44\x65tectedFramework\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07version\x18\x02 \x01(\t\x12\x10\n\x08language\x18\x03 \x01(\t\x12\x12\n\nconfidence\x18\x04 \x01(\x02\x12\x15\n\rdetected_from\x18\x05 \x01(\t\"\xce\x02\n\x0cProjectStats\x12\x13\n\x0btotal_files\x18\x01 \x01(\x05\x12\x13\n\x0btotal_lines\x18\x02 \x01(\x05\x12\x18\n\x10total_size_bytes\x18\x03 \x01(\x03\x12\x44\n\x11\x66iles_by_language\x18\x04 \x03(\x0b\x32).warden.ProjectStats.FilesByLanguageEntry\x12\x44\n\x11lines_by_language\x18\x05 \x03(\x0b\x32).warden.ProjectStats.LinesByLanguageEntry\x1a\x36\n\x14\x46ilesByLanguageEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x05:\x02\x38\x01\x1a\x36\n\x14LinesByLanguageEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x05:\x02\x38\x01\"{\n\x11LlmAnalyzeRequest\x12\x0c\n\x04\x63ode\x18\x01 \x01(\t\x12\x0e\n\x06prompt\x18\x02 \x01(\t\x12\x10\n\x08provider\x18\x03 \x01(\t\x12\r\n\x05model\x18\x04 \x01(\t\x12\x13\n\x0btemperature\x18\x05 \x01(\x02\x12\x12\n\nmax_tokens\x18\x06 \x01(\x05\"\xab\x01\n\x10LlmAnalyzeResult\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x10\n\x08response\x18\x02 \x01(\t\x12\x15\n\rprovider_used\x18\x03 \x01(\t\x12\x12\n\nmodel_used\x18\x04 \x01(\t\x12\x13\n\x0btokens_used\x18\x05 \x01(\x05\x12\x13\n\x0b\x64uration_ms\x18\x06 \x01(\x03\x12\x1f\n\x05\x65rror\x18\x07 \x01(\x0b\x32\x10.warden.LlmError\";\n\x08LlmError\x12\x0c\n\x04\x63ode\x18\x01 \x01(\t\x12\x0f\n\x07message\x18\x02 \x01(\t\x12\x10\n\x08provider\x18\x03 \x01(\t\"7\n\x13TestProviderRequest\x12\x10\n\x08provider\x18\x01 \x01(\t\x12\x0e\n\x06prompt\x18\x02 \x01(\t\"s\n\x14TestProviderResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x10\n\x08provider\x18\x02 \x01(\t\x12\r\n\x05model\x18\x03 \x01(\t\x12\x12\n\nlatency_ms\x18\x04 \x01(\x03\x12\x15\n\rerror_message\x18\x05 \x01(\t\"*\n\x16\x41vailableModelsRequest\x12\x10\n\x08provider\x18\x01 \x01(\t\";\n\x17\x41vailableModelsResponse\x12 \n\x06models\x18\x01 \x03(\x0b\x32\x10.warden.LlmModel\"]\n\x08LlmModel\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x10\n\x08provider\x18\x03 \x01(\t\x12\x12\n\nmax_tokens\x18\x04 \x01(\x05\x12\x11\n\tavailable\x18\x05 \x01(\x08\"\x99\x01\n\x18ValidateLlmConfigRequest\x12\x10\n\x08provider\x18\x01 \x01(\t\x12<\n\x06\x63onfig\x18\x02 \x03(\x0b\x32,.warden.ValidateLlmConfigRequest.ConfigEntry\x1a-\n\x0b\x43onfigEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"L\n\x19ValidateLlmConfigResponse\x12\r\n\x05valid\x18\x01 \x01(\x08\x12\x0e\n\x06\x65rrors\x18\x02 \x03(\t\x12\x10\n\x08warnings\x18\x03 \x03(\t\"1\n\x0e\x43leanupRequest\x12\x0c\n\x04path\x18\x01 \x01(\t\x12\x11\n\tanalyzers\x18\x02 \x03(\t\"\x93\x01\n\rCleanupResult\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12.\n\x0bsuggestions\x18\x02 \x03(\x0b\x32\x19.warden.CleanupSuggestion\x12\x15\n\rcleanup_score\x18\x03 \x01(\x02\x12\x13\n\x0b\x64uration_ms\x18\x04 \x01(\x03\x12\x15\n\rerror_message\x18\x05 \x01(\t\"\xce\x01\n\x11\x43leanupSuggestion\x12\n\n\x02id\x18\x01 \x01(\t\x12\x10\n\x08\x61nalyzer\x18\x02 \x01(\t\x12\r\n\x05title\x18\x03 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x04 \x01(\t\x12\x11\n\tfile_path\x18\x05 \x01(\t\x12\x13\n\x0bline_number\x18\x06 \x01(\x05\x12\x14\n\x0c\x63ode_snippet\x18\x07 \x01(\t\x12\x15\n\rsuggested_fix\x18\x08 \x01(\t\x12\"\n\x08priority\x18\t \x01(\x0e\x32\x10.warden.Severity\"\xbe\x01\n\x14\x43leanupScoreResponse\x12\x15\n\roverall_score\x18\x01 \x01(\x02\x12I\n\x0f\x61nalyzer_scores\x18\x02 \x03(\x0b\x32\x30.warden.CleanupScoreResponse.AnalyzerScoresEntry\x12\r\n\x05grade\x18\x03 \x01(\t\x1a\x35\n\x13\x41nalyzerScoresEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x02:\x02\x38\x01\"8\n\x14\x46ortificationRequest\x12\x0c\n\x04path\x18\x01 \x01(\t\x12\x12\n\nfortifiers\x18\x02 \x03(\t\"\xa0\x01\n\x13\x46ortificationResult\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x34\n\x0bsuggestions\x18\x02 \x03(\x0b\x32\x1f.warden.FortificationSuggestion\x12\x16\n\x0esecurity_score\x18\x03 \x01(\x02\x12\x13\n\x0b\x64uration_ms\x18\x04 \x01(\x03\x12\x15\n\rerror_message\x18\x05 \x01(\t\"\xea\x01\n\x17\x46ortificationSuggestion\x12\n\n\x02id\x18\x01 \x01(\t\x12\x11\n\tfortifier\x18\x02 \x01(\t\x12\r\n\x05title\x18\x03 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x04 \x01(\t\x12\x11\n\tfile_path\x18\x05 \x01(\t\x12\x13\n\x0bline_number\x18\x06 \x01(\x05\x12\x15\n\roriginal_code\x18\x07 \x01(\t\x12\x16\n\x0esuggested_code\x18\x08 \x01(\t\x12\x11\n\trationale\x18\t \x01(\t\x12\"\n\x08priority\x18\n \x01(\x0e\x32\x10.warden.Severity\"C\n\x19\x41pplyFortificationRequest\x12\x15\n\rsuggestion_id\x18\x01 \x01(\t\x12\x0f\n\x07\x64ry_run\x18\x02 \x01(\x08\"e\n\x1a\x41pplyFortificationResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x11\n\tfile_path\x18\x02 \x01(\t\x12\x0c\n\x04\x64iff\x18\x03 \x01(\t\x12\x15\n\rerror_message\x18\x04 \x01(\t\"\xd9\x01\n\x15SecurityScoreResponse\x12\x15\n\roverall_score\x18\x01 \x01(\x02\x12J\n\x0f\x63\x61tegory_scores\x18\x02 \x03(\x0b\x32\x31.warden.SecurityScoreResponse.CategoryScoresEntry\x12\x17\n\x0fvulnerabilities\x18\x03 \x03(\t\x12\r\n\x05grade\x18\x04 \x01(\t\x1a\x35\n\x13\x43\x61tegoryScoresEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x02:\x02\x38\x01\"\xb8\x01\n\x0bSuppression\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0f\n\x07rule_id\x18\x02 \x01(\t\x12\x11\n\tfile_path\x18\x03 \x01(\t\x12\x13\n\x0bline_number\x18\x04 \x01(\x05\x12\x15\n\rjustification\x18\x05 \x01(\t\x12\x12\n\ncreated_by\x18\x06 \x01(\t\x12\x12\n\ncreated_at\x18\x07 \x01(\t\x12\x12\n\nexpires_at\x18\x08 \x01(\t\x12\x11\n\tis_global\x18\t \x01(\x08\"\xa2\x01\n\x15\x41\x64\x64SuppressionRequest\x12\x0f\n\x07rule_id\x18\x01 \x01(\t\x12\x11\n\tfile_path\x18\x02 \x01(\t\x12\x13\n\x0bline_number\x18\x03 \x01(\x05\x12\x15\n\rjustification\x18\x04 \x01(\t\x12\x12\n\ncreated_by\x18\x05 \x01(\t\x12\x12\n\nexpires_at\x18\x06 \x01(\t\x12\x11\n\tis_global\x18\x07 \x01(\x08\"j\n\x16\x41\x64\x64SuppressionResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12(\n\x0bsuppression\x18\x02 \x01(\x0b\x32\x13.warden.Suppression\x12\x15\n\rerror_message\x18\x03 \x01(\t\"2\n\x18RemoveSuppressionRequest\x12\x16\n\x0esuppression_id\x18\x01 \x01(\t\"C\n\x19RemoveSuppressionResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x15\n\rerror_message\x18\x02 \x01(\t\"Q\n\x0fSuppressionList\x12)\n\x0csuppressions\x18\x01 \x03(\x0b\x32\x13.warden.Suppression\x12\x13\n\x0btotal_count\x18\x02 \x01(\x05\"R\n\x17\x43heckSuppressionRequest\x12\x0f\n\x07rule_id\x18\x01 \x01(\t\x12\x11\n\tfile_path\x18\x02 \x01(\t\x12\x13\n\x0bline_number\x18\x03 \x01(\x05\"k\n\x18\x43heckSuppressionResponse\x12\x15\n\ris_suppressed\x18\x01 \x01(\x08\x12(\n\x0bsuppression\x18\x02 \x01(\x0b\x32\x13.warden.Suppression\x12\x0e\n\x06reason\x18\x03 \x01(\t\"\x83\x01\n\x13UpdateConfigRequest\x12;\n\x08settings\x18\x01 \x03(\x0b\x32).warden.UpdateConfigRequest.SettingsEntry\x1a/\n\rSettingsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\">\n\x14UpdateConfigResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x15\n\rerror_message\x18\x02 \x01(\t\"=\n\x18UpdateFrameStatusRequest\x12\x10\n\x08\x66rame_id\x18\x01 \x01(\t\x12\x0f\n\x07\x65nabled\x18\x02 \x01(\x08\"a\n\x19UpdateFrameStatusResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x1c\n\x05\x66rame\x18\x02 \x01(\x0b\x32\r.warden.Frame\x12\x15\n\rerror_message\x18\x03 \x01(\t\"2\n\x0f\x43lassifyRequest\x12\x0c\n\x04\x63ode\x18\x01 \x01(\t\x12\x11\n\tfile_path\x18\x02 \x01(\t\"\xa2\x02\n\x0e\x43lassifyResult\x12\x1c\n\x14has_async_operations\x18\x01 \x01(\x08\x12\x16\n\x0ehas_user_input\x18\x02 \x01(\x08\x12\x1f\n\x17has_database_operations\x18\x03 \x01(\x08\x12\x19\n\x11has_network_calls\x18\x04 \x01(\x08\x12\x1b\n\x13has_file_operations\x18\x05 \x01(\x08\x12\x1a\n\x12has_authentication\x18\x06 \x01(\x08\x12\x18\n\x10has_cryptography\x18\x07 \x01(\x08\x12\x1b\n\x13\x64\x65tected_frameworks\x18\x08 \x03(\t\x12\x1a\n\x12recommended_frames\x18\t \x03(\t\x12\x12\n\nconfidence\x18\n \x01(\x02\"{\n\x05\x46rame\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x03 \x01(\t\x12\x10\n\x08priority\x18\x04 \x01(\x05\x12\x12\n\nis_blocker\x18\x05 \x01(\x08\x12\x0f\n\x07\x65nabled\x18\x06 \x01(\x08\x12\x0c\n\x04tags\x18\x07 \x03(\t\"*\n\tFrameList\x12\x1d\n\x06\x66rames\x18\x01 \x03(\x0b\x32\r.warden.Frame\"[\n\x08Provider\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x11\n\tavailable\x18\x03 \x01(\x08\x12\x12\n\nis_default\x18\x04 \x01(\x08\x12\x0e\n\x06status\x18\x05 \x01(\t\"M\n\x0cProviderList\x12#\n\tproviders\x18\x01 \x03(\x0b\x32\x10.warden.Provider\x12\x18\n\x10\x64\x65\x66\x61ult_provider\x18\x02 \x01(\t\"\xaa\x02\n\x15\x43onfigurationResponse\x12\x14\n\x0cproject_root\x18\x01 \x01(\t\x12\x13\n\x0b\x63onfig_file\x18\x02 \x01(\t\x12\x16\n\x0e\x61\x63tive_profile\x18\x03 \x01(\t\x12+\n\x10\x61vailable_frames\x18\x04 \x01(\x0b\x32\x11.warden.FrameList\x12\x31\n\x13\x61vailable_providers\x18\x05 \x01(\x0b\x32\x14.warden.ProviderList\x12=\n\x08settings\x18\x06 \x03(\x0b\x32+.warden.ConfigurationResponse.SettingsEntry\x1a/\n\rSettingsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\xb9\x01\n\x0eHealthResponse\x12\x0f\n\x07healthy\x18\x01 \x01(\x08\x12\x0f\n\x07version\x18\x02 \x01(\t\x12\x16\n\x0euptime_seconds\x18\x03 \x01(\x03\x12:\n\ncomponents\x18\x04 \x03(\x0b\x32&.warden.HealthResponse.ComponentsEntry\x1a\x31\n\x0f\x43omponentsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x08:\x02\x38\x01\"\x90\x01\n\x0eStatusResponse\x12\x0f\n\x07running\x18\x01 \x01(\x08\x12\x18\n\x10\x61\x63tive_pipelines\x18\x02 \x01(\x05\x12\x13\n\x0btotal_scans\x18\x03 \x01(\x03\x12\x16\n\x0etotal_findings\x18\x04 \x01(\x03\x12\x11\n\tmemory_mb\x18\x05 \x01(\x03\x12\x13\n\x0b\x63pu_percent\x18\x06 \x01(\x02\"\xc1\x01\n\x0fPipelineRequest\x12\x0c\n\x04path\x18\x01 \x01(\t\x12\x0e\n\x06\x66rames\x18\x02 \x03(\t\x12\x10\n\x08parallel\x18\x03 \x01(\x08\x12\x17\n\x0ftimeout_seconds\x18\x04 \x01(\x05\x12\x35\n\x07options\x18\x05 \x03(\x0b\x32$.warden.PipelineRequest.OptionsEntry\x1a.\n\x0cOptionsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\xda\x02\n\x0ePipelineResult\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x0e\n\x06run_id\x18\x02 \x01(\t\x12\x16\n\x0etotal_findings\x18\x03 \x01(\x05\x12\x16\n\x0e\x63ritical_count\x18\x04 \x01(\x05\x12\x12\n\nhigh_count\x18\x05 \x01(\x05\x12\x14\n\x0cmedium_count\x18\x06 \x01(\x05\x12\x11\n\tlow_count\x18\x07 \x01(\x05\x12!\n\x08\x66indings\x18\x08 \x03(\x0b\x32\x0f.warden.Finding\x12-\n\x0e\x66ortifications\x18\t \x03(\x0b\x32\x15.warden.Fortification\x12#\n\tcleanings\x18\n \x03(\x0b\x32\x10.warden.Cleaning\x12\x13\n\x0b\x64uration_ms\x18\x0b \x01(\x03\x12\x17\n\x0f\x66rames_executed\x18\x0c \x03(\t\x12\x15\n\rerror_message\x18\r \x01(\t\"\xdf\x01\n\rPipelineEvent\x12\x12\n\nevent_type\x18\x01 \x01(\t\x12\r\n\x05stage\x18\x02 \x01(\t\x12\x10\n\x08progress\x18\x03 \x01(\x02\x12\x0f\n\x07message\x18\x04 \x01(\t\x12 \n\x07\x66inding\x18\x05 \x01(\x0b\x32\x0f.warden.Finding\x12,\n\rfortification\x18\x06 \x01(\x0b\x32\x15.warden.Fortification\x12\"\n\x08\x63leaning\x18\x07 \x01(\x0b\x32\x10.warden.Cleaning\x12\x14\n\x0ctimestamp_ms\x18\x08 \x01(\x03*[\n\x08Severity\x12\x18\n\x14SEVERITY_UNSPECIFIED\x10\x00\x12\x0c\n\x08\x43RITICAL\x10\x01\x12\x08\n\x04HIGH\x10\x02\x12\n\n\x06MEDIUM\x10\x03\x12\x07\n\x03LOW\x10\x04\x12\x08\n\x04INFO\x10\x05*_\n\nIssueState\x12\x1b\n\x17ISSUE_STATE_UNSPECIFIED\x10\x00\x12\x08\n\x04OPEN\x10\x01\x12\x0c\n\x08RESOLVED\x10\x02\x12\x0e\n\nSUPPRESSED\x10\x03\x12\x0c\n\x08REOPENED\x10\x04*Q\n\x0eTrendDirection\x12\x15\n\x11TREND_UNSPECIFIED\x10\x00\x12\r\n\tIMPROVING\x10\x01\x12\n\n\x06STABLE\x10\x02\x12\r\n\tDEGRADING\x10\x03*\xc2\x01\n\x08\x46ileType\x12\x19\n\x15\x46ILE_TYPE_UNSPECIFIED\x10\x00\x12\n\n\x06PYTHON\x10\x01\x12\x0e\n\nJAVASCRIPT\x10\x02\x12\x0e\n\nTYPESCRIPT\x10\x03\x12\x08\n\x04JAVA\x10\x04\x12\n\n\x06\x43SHARP\x10\x05\x12\x06\n\x02GO\x10\x06\x12\x08\n\x04RUST\x10\x07\x12\x07\n\x03\x43PP\x10\x08\x12\x08\n\x04RUBY\x10\t\x12\x07\n\x03PHP\x10\n\x12\n\n\x06KOTLIN\x10\x0b\x12\t\n\x05SWIFT\x10\x0c\x12\t\n\x05SCALA\x10\r\x12\t\n\x05OTHER\x10\x63\x32\xa4\x1b\n\rWardenService\x12\x42\n\x0f\x45xecutePipeline\x12\x17.warden.PipelineRequest\x1a\x16.warden.PipelineResult\x12I\n\x15\x45xecutePipelineStream\x12\x17.warden.PipelineRequest\x1a\x15.warden.PipelineEvent0\x01\x12\x36\n\x0cGetAllIssues\x12\x13.warden.IssueFilter\x1a\x11.warden.IssueList\x12\x37\n\rGetOpenIssues\x12\x13.warden.IssueFilter\x1a\x11.warden.IssueList\x12\x39\n\x0eGetIssueByHash\x12\x18.warden.IssueHashRequest\x1a\r.warden.Issue\x12G\n\x0cResolveIssue\x12\x1a.warden.IssueActionRequest\x1a\x1b.warden.IssueActionResponse\x12H\n\rSuppressIssue\x12\x1a.warden.IssueActionRequest\x1a\x1b.warden.IssueActionResponse\x12\x46\n\x0bReopenIssue\x12\x1a.warden.IssueActionRequest\x1a\x1b.warden.IssueActionResponse\x12\x36\n\x0fGetIssueHistory\x12\r.warden.Empty\x1a\x14.warden.IssueHistory\x12\x32\n\rGetIssueStats\x12\r.warden.Empty\x1a\x12.warden.IssueStats\x12G\n\x0e\x41nalyzeResults\x12\x1d.warden.AnalyzeResultsRequest\x1a\x16.warden.AnalysisResult\x12\x38\n\tGetTrends\x12\x14.warden.TrendRequest\x1a\x15.warden.TrendResponse\x12\x32\n\rGetFrameStats\x12\r.warden.Empty\x1a\x12.warden.FrameStats\x12\x38\n\x10GetSeverityStats\x12\r.warden.Empty\x1a\x15.warden.SeverityStats\x12>\n\x0fGetQualityScore\x12\r.warden.Empty\x1a\x1c.warden.QualityScoreResponse\x12\x43\n\x12GenerateHtmlReport\x12\x15.warden.ReportRequest\x1a\x16.warden.ReportResponse\x12\x42\n\x11GeneratePdfReport\x12\x15.warden.ReportRequest\x1a\x16.warden.ReportResponse\x12\x43\n\x12GenerateJsonReport\x12\x15.warden.ReportRequest\x1a\x16.warden.ReportResponse\x12L\n\x0fGetReportStatus\x12\x1b.warden.ReportStatusRequest\x1a\x1c.warden.ReportStatusResponse\x12\x39\n\nSearchCode\x12\x15.warden.SearchRequest\x1a\x14.warden.SearchResult\x12\x45\n\x11SearchSimilarCode\x12\x1a.warden.SimilarCodeRequest\x1a\x14.warden.SearchResult\x12\x42\n\x13SearchByDescription\x12\x15.warden.SearchRequest\x1a\x14.warden.SearchResult\x12;\n\x0cIndexProject\x12\x14.warden.IndexRequest\x1a\x15.warden.IndexResponse\x12\x32\n\rGetIndexStats\x12\r.warden.Empty\x1a\x12.warden.IndexStats\x12\x32\n\nClearIndex\x12\r.warden.Empty\x1a\x15.warden.IndexResponse\x12\x42\n\rDiscoverFiles\x12\x17.warden.DiscoverRequest\x1a\x18.warden.DiscoverResponse\x12\x42\n\x0eGetFilesByType\x12\x16.warden.FileTypeFilter\x1a\x18.warden.DiscoverResponse\x12\x45\n\x10\x44\x65tectFrameworks\x12\x17.warden.DiscoverRequest\x1a\x18.warden.DiscoverResponse\x12\x36\n\x0fGetProjectStats\x12\r.warden.Empty\x1a\x14.warden.ProjectStats\x12\x45\n\x0e\x41nalyzeWithLlm\x12\x19.warden.LlmAnalyzeRequest\x1a\x18.warden.LlmAnalyzeResult\x12?\n\x0c\x43lassifyCode\x12\x17.warden.ClassifyRequest\x1a\x16.warden.ClassifyResult\x12L\n\x0fTestLlmProvider\x12\x1b.warden.TestProviderRequest\x1a\x1c.warden.TestProviderResponse\x12U\n\x12GetAvailableModels\x12\x1e.warden.AvailableModelsRequest\x1a\x1f.warden.AvailableModelsResponse\x12X\n\x11ValidateLlmConfig\x12 .warden.ValidateLlmConfigRequest\x1a!.warden.ValidateLlmConfigResponse\x12?\n\x0e\x41nalyzeCleanup\x12\x16.warden.CleanupRequest\x1a\x15.warden.CleanupResult\x12\x46\n\x15GetCleanupSuggestions\x12\x16.warden.CleanupRequest\x1a\x15.warden.CleanupResult\x12>\n\x0fGetCleanupScore\x12\r.warden.Empty\x1a\x1c.warden.CleanupScoreResponse\x12X\n\x1bGetFortificationSuggestions\x12\x1c.warden.FortificationRequest\x1a\x1b.warden.FortificationResult\x12[\n\x12\x41pplyFortification\x12!.warden.ApplyFortificationRequest\x1a\".warden.ApplyFortificationResponse\x12@\n\x10GetSecurityScore\x12\r.warden.Empty\x1a\x1d.warden.SecurityScoreResponse\x12O\n\x0e\x41\x64\x64Suppression\x12\x1d.warden.AddSuppressionRequest\x1a\x1e.warden.AddSuppressionResponse\x12X\n\x11RemoveSuppression\x12 .warden.RemoveSuppressionRequest\x1a!.warden.RemoveSuppressionResponse\x12\x39\n\x0fGetSuppressions\x12\r.warden.Empty\x1a\x17.warden.SuppressionList\x12U\n\x10\x43heckSuppression\x12\x1f.warden.CheckSuppressionRequest\x1a .warden.CheckSuppressionResponse\x12\x36\n\x12GetAvailableFrames\x12\r.warden.Empty\x1a\x11.warden.FrameList\x12<\n\x15GetAvailableProviders\x12\r.warden.Empty\x1a\x14.warden.ProviderList\x12@\n\x10GetConfiguration\x12\r.warden.Empty\x1a\x1d.warden.ConfigurationResponse\x12P\n\x13UpdateConfiguration\x12\x1b.warden.UpdateConfigRequest\x1a\x1c.warden.UpdateConfigResponse\x12X\n\x11UpdateFrameStatus\x12 .warden.UpdateFrameStatusRequest\x1a!.warden.UpdateFrameStatusResponse\x12\x34\n\x0bHealthCheck\x12\r.warden.Empty\x1a\x16.warden.HealthResponse\x12\x32\n\tGetStatus\x12\r.warden.Empty\x1a\x16.warden.StatusResponseB\x0e\xaa\x02\x0bWarden.Grpcb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'warden_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\252\002\013Warden.Grpc'
  _globals['_FINDING_METADATAENTRY']._loaded_options = None
  _globals['_FINDING_METADATAENTRY']._serialized_options = b'8\001'
  _globals['_ISSUE_METADATAENTRY']._loaded_options = None
  _globals['_ISSUE_METADATAENTRY']._serialized_options = b'8\001'
  _globals['_ISSUESTATS_BYFRAMEENTRY']._loaded_options = None
  _globals['_ISSUESTATS_BYFRAMEENTRY']._serialized_options = b'8\001'
  _globals['_ISSUESTATS_BYFILEENTRY']._loaded_options = None
  _globals['_ISSUESTATS_BYFILEENTRY']._serialized_options = b'8\001'
  _globals['_FRAMESTATS_STATSENTRY']._loaded_options = None
  _globals['_FRAMESTATS_STATSENTRY']._serialized_options = b'8\001'
  _globals['_QUALITYSCORERESPONSE_BREAKDOWNENTRY']._loaded_options = None
  _globals['_QUALITYSCORERESPONSE_BREAKDOWNENTRY']._serialized_options = b'8\001'
  _globals['_REPORTREQUEST_METADATAENTRY']._loaded_options = None
  _globals['_REPORTREQUEST_METADATAENTRY']._serialized_options = b'8\001'
  _globals['_CODECHUNK_METADATAENTRY']._loaded_options = None
  _globals['_CODECHUNK_METADATAENTRY']._serialized_options = b'8\001'
  _globals['_INDEXSTATS_CHUNKSBYLANGUAGEENTRY']._loaded_options = None
  _globals['_INDEXSTATS_CHUNKSBYLANGUAGEENTRY']._serialized_options = b'8\001'
  _globals['_INDEXSTATS_CHUNKSBYTYPEENTRY']._loaded_options = None
  _globals['_INDEXSTATS_CHUNKSBYTYPEENTRY']._serialized_options = b'8\001'
  _globals['_PROJECTSTATS_FILESBYLANGUAGEENTRY']._loaded_options = None
  _globals['_PROJECTSTATS_FILESBYLANGUAGEENTRY']._serialized_options = b'8\001'
  _globals['_PROJECTSTATS_LINESBYLANGUAGEENTRY']._loaded_options = None
  _globals['_PROJECTSTATS_LINESBYLANGUAGEENTRY']._serialized_options = b'8\001'
  _globals['_VALIDATELLMCONFIGREQUEST_CONFIGENTRY']._loaded_options = None
  _globals['_VALIDATELLMCONFIGREQUEST_CONFIGENTRY']._serialized_options = b'8\001'
  _globals['_CLEANUPSCORERESPONSE_ANALYZERSCORESENTRY']._loaded_options = None
  _globals['_CLEANUPSCORERESPONSE_ANALYZERSCORESENTRY']._serialized_options = b'8\001'
  _globals['_SECURITYSCORERESPONSE_CATEGORYSCORESENTRY']._loaded_options = None
  _globals['_SECURITYSCORERESPONSE_CATEGORYSCORESENTRY']._serialized_options = b'8\001'
  _globals['_UPDATECONFIGREQUEST_SETTINGSENTRY']._loaded_options = None
  _globals['_UPDATECONFIGREQUEST_SETTINGSENTRY']._serialized_options = b'8\001'
  _globals['_CONFIGURATIONRESPONSE_SETTINGSENTRY']._loaded_options = None
  _globals['_CONFIGURATIONRESPONSE_SETTINGSENTRY']._serialized_options = b'8\001'
  _globals['_HEALTHRESPONSE_COMPONENTSENTRY']._loaded_options = None
  _globals['_HEALTHRESPONSE_COMPONENTSENTRY']._serialized_options = b'8\001'
  _globals['_PIPELINEREQUEST_OPTIONSENTRY']._loaded_options = None
  _globals['_PIPELINEREQUEST_OPTIONSENTRY']._serialized_options = b'8\001'
  _globals['_SEVERITY']._serialized_start=12086
  _globals['_SEVERITY']._serialized_end=12177
  _globals['_ISSUESTATE']._serialized_start=12179
  _globals['_ISSUESTATE']._serialized_end=12274
  _globals['_TRENDDIRECTION']._serialized_start=12276
  _globals['_TRENDDIRECTION']._serialized_end=12357
  _globals['_FILETYPE']._serialized_start=12360
  _globals['_FILETYPE']._serialized_end=12554
  _globals['_EMPTY']._serialized_start=24
  _globals['_EMPTY']._serialized_end=31
  _globals['_FINDING']._serialized_start=34
  _globals['_FINDING']._serialized_end=388
  _globals['_FINDING_METADATAENTRY']._serialized_start=341
  _globals['_FINDING_METADATAENTRY']._serialized_end=388
  _globals['_FORTIFICATION']._serialized_start=391
  _globals['_FORTIFICATION']._serialized_end=596
  _globals['_CLEANING']._serialized_start=598
  _globals['_CLEANING']._serialized_end=712
  _globals['_ISSUE']._serialized_start=715
  _globals['_ISSUE']._serialized_end=1216
  _globals['_ISSUE_METADATAENTRY']._serialized_start=341
  _globals['_ISSUE_METADATAENTRY']._serialized_end=388
  _globals['_ISSUEFILTER']._serialized_start=1219
  _globals['_ISSUEFILTER']._serialized_end=1383
  _globals['_ISSUELIST']._serialized_start=1385
  _globals['_ISSUELIST']._serialized_end=1472
  _globals['_ISSUEREQUEST']._serialized_start=1474
  _globals['_ISSUEREQUEST']._serialized_end=1506
  _globals['_ISSUEHASHREQUEST']._serialized_start=1508
  _globals['_ISSUEHASHREQUEST']._serialized_end=1540
  _globals['_ISSUEACTIONREQUEST']._serialized_start=1542
  _globals['_ISSUEACTIONREQUEST']._serialized_end=1612
  _globals['_ISSUEACTIONRESPONSE']._serialized_start=1614
  _globals['_ISSUEACTIONRESPONSE']._serialized_end=1705
  _globals['_ISSUESTATS']._serialized_start=1708
  _globals['_ISSUESTATS']._serialized_end=2057
  _globals['_ISSUESTATS_BYFRAMEENTRY']._serialized_start=1964
  _globals['_ISSUESTATS_BYFRAMEENTRY']._serialized_end=2010
  _globals['_ISSUESTATS_BYFILEENTRY']._serialized_start=2012
  _globals['_ISSUESTATS_BYFILEENTRY']._serialized_end=2057
  _globals['_ISSUEHISTORY']._serialized_start=2059
  _globals['_ISSUEHISTORY']._serialized_end=2115
  _globals['_ISSUESNAPSHOT']._serialized_start=2118
  _globals['_ISSUESNAPSHOT']._serialized_end=2312
  _globals['_ANALYZERESULTSREQUEST']._serialized_start=2314
  _globals['_ANALYZERESULTSREQUEST']._serialized_end=2434
  _globals['_ANALYSISRESULT']._serialized_start=2437
  _globals['_ANALYSISRESULT']._serialized_end=2793
  _globals['_TRENDINGISSUE']._serialized_start=2795
  _globals['_TRENDINGISSUE']._serialized_end=2910
  _globals['_FRAMESTATS']._serialized_start=2912
  _globals['_FRAMESTATS']._serialized_end=3035
  _globals['_FRAMESTATS_STATSENTRY']._serialized_start=2972
  _globals['_FRAMESTATS_STATSENTRY']._serialized_end=3035
  _globals['_FRAMESTAT']._serialized_start=3038
  _globals['_FRAMESTAT']._serialized_end=3221
  _globals['_SEVERITYSTATS']._serialized_start=3223
  _globals['_SEVERITYSTATS']._serialized_end=3337
  _globals['_TRENDREQUEST']._serialized_start=3339
  _globals['_TRENDREQUEST']._serialized_end=3368
  _globals['_TRENDRESPONSE']._serialized_start=3370
  _globals['_TRENDRESPONSE']._serialized_end=3468
  _globals['_TRENDPOINT']._serialized_start=3470
  _globals['_TRENDPOINT']._serialized_end=3546
  _globals['_QUALITYSCORERESPONSE']._serialized_start=3549
  _globals['_QUALITYSCORERESPONSE']._serialized_end=3715
  _globals['_QUALITYSCORERESPONSE_BREAKDOWNENTRY']._serialized_start=3667
  _globals['_QUALITYSCORERESPONSE_BREAKDOWNENTRY']._serialized_end=3715
  _globals['_REPORTREQUEST']._serialized_start=3718
  _globals['_REPORTREQUEST']._serialized_end=3933
  _globals['_REPORTREQUEST_METADATAENTRY']._serialized_start=341
  _globals['_REPORTREQUEST_METADATAENTRY']._serialized_end=388
  _globals['_REPORTRESPONSE']._serialized_start=3936
  _globals['_REPORTRESPONSE']._serialized_end=4067
  _globals['_REPORTSTATUSREQUEST']._serialized_start=4069
  _globals['_REPORTSTATUSREQUEST']._serialized_end=4109
  _globals['_REPORTSTATUSRESPONSE']._serialized_start=4111
  _globals['_REPORTSTATUSRESPONSE']._serialized_end=4228
  _globals['_SEARCHREQUEST']._serialized_start=4230
  _globals['_SEARCHREQUEST']._serialized_end=4333
  _globals['_SEARCHRESULT']._serialized_start=4335
  _globals['_SEARCHRESULT']._serialized_end=4432
  _globals['_CODECHUNK']._serialized_start=4435
  _globals['_CODECHUNK']._serialized_end=4710
  _globals['_CODECHUNK_METADATAENTRY']._serialized_start=341
  _globals['_CODECHUNK_METADATAENTRY']._serialized_end=388
  _globals['_SIMILARCODEREQUEST']._serialized_start=4712
  _globals['_SIMILARCODEREQUEST']._serialized_end=4798
  _globals['_INDEXREQUEST']._serialized_start=4800
  _globals['_INDEXREQUEST']._serialized_end=4870
  _globals['_INDEXRESPONSE']._serialized_start=4872
  _globals['_INDEXRESPONSE']._serialized_end=4995
  _globals['_INDEXSTATS']._serialized_start=4998
  _globals['_INDEXSTATS']._serialized_end=5343
  _globals['_INDEXSTATS_CHUNKSBYLANGUAGEENTRY']._serialized_start=5235
  _globals['_INDEXSTATS_CHUNKSBYLANGUAGEENTRY']._serialized_end=5290
  _globals['_INDEXSTATS_CHUNKSBYTYPEENTRY']._serialized_start=5292
  _globals['_INDEXSTATS_CHUNKSBYTYPEENTRY']._serialized_end=5343
  _globals['_DISCOVERREQUEST']._serialized_start=5345
  _globals['_DISCOVERREQUEST']._serialized_end=5470
  _globals['_DISCOVERRESPONSE']._serialized_start=5473
  _globals['_DISCOVERRESPONSE']._serialized_end=5701
  _globals['_DISCOVEREDFILE']._serialized_start=5704
  _globals['_DISCOVEREDFILE']._serialized_end=5852
  _globals['_FILETYPEFILTER']._serialized_start=5854
  _globals['_FILETYPEFILTER']._serialized_end=5903
  _globals['_DETECTEDFRAMEWORK']._serialized_start=5905
  _globals['_DETECTEDFRAMEWORK']._serialized_end=6016
  _globals['_PROJECTSTATS']._serialized_start=6019
  _globals['_PROJECTSTATS']._serialized_end=6353
  _globals['_PROJECTSTATS_FILESBYLANGUAGEENTRY']._serialized_start=6243
  _globals['_PROJECTSTATS_FILESBYLANGUAGEENTRY']._serialized_end=6297
  _globals['_PROJECTSTATS_LINESBYLANGUAGEENTRY']._serialized_start=6299
  _globals['_PROJECTSTATS_LINESBYLANGUAGEENTRY']._serialized_end=6353
  _globals['_LLMANALYZEREQUEST']._serialized_start=6355
  _globals['_LLMANALYZEREQUEST']._serialized_end=6478
  _globals['_LLMANALYZERESULT']._serialized_start=6481
  _globals['_LLMANALYZERESULT']._serialized_end=6652
  _globals['_LLMERROR']._serialized_start=6654
  _globals['_LLMERROR']._serialized_end=6713
  _globals['_TESTPROVIDERREQUEST']._serialized_start=6715
  _globals['_TESTPROVIDERREQUEST']._serialized_end=6770
  _globals['_TESTPROVIDERRESPONSE']._serialized_start=6772
  _globals['_TESTPROVIDERRESPONSE']._serialized_end=6887
  _globals['_AVAILABLEMODELSREQUEST']._serialized_start=6889
  _globals['_AVAILABLEMODELSREQUEST']._serialized_end=6931
  _globals['_AVAILABLEMODELSRESPONSE']._serialized_start=6933
  _globals['_AVAILABLEMODELSRESPONSE']._serialized_end=6992
  _globals['_LLMMODEL']._serialized_start=6994
  _globals['_LLMMODEL']._serialized_end=7087
  _globals['_VALIDATELLMCONFIGREQUEST']._serialized_start=7090
  _globals['_VALIDATELLMCONFIGREQUEST']._serialized_end=7243
  _globals['_VALIDATELLMCONFIGREQUEST_CONFIGENTRY']._serialized_start=7198
  _globals['_VALIDATELLMCONFIGREQUEST_CONFIGENTRY']._serialized_end=7243
  _globals['_VALIDATELLMCONFIGRESPONSE']._serialized_start=7245
  _globals['_VALIDATELLMCONFIGRESPONSE']._serialized_end=7321
  _globals['_CLEANUPREQUEST']._serialized_start=7323
  _globals['_CLEANUPREQUEST']._serialized_end=7372
  _globals['_CLEANUPRESULT']._serialized_start=7375
  _globals['_CLEANUPRESULT']._serialized_end=7522
  _globals['_CLEANUPSUGGESTION']._serialized_start=7525
  _globals['_CLEANUPSUGGESTION']._serialized_end=7731
  _globals['_CLEANUPSCORERESPONSE']._serialized_start=7734
  _globals['_CLEANUPSCORERESPONSE']._serialized_end=7924
  _globals['_CLEANUPSCORERESPONSE_ANALYZERSCORESENTRY']._serialized_start=7871
  _globals['_CLEANUPSCORERESPONSE_ANALYZERSCORESENTRY']._serialized_end=7924
  _globals['_FORTIFICATIONREQUEST']._serialized_start=7926
  _globals['_FORTIFICATIONREQUEST']._serialized_end=7982
  _globals['_FORTIFICATIONRESULT']._serialized_start=7985
  _globals['_FORTIFICATIONRESULT']._serialized_end=8145
  _globals['_FORTIFICATIONSUGGESTION']._serialized_start=8148
  _globals['_FORTIFICATIONSUGGESTION']._serialized_end=8382
  _globals['_APPLYFORTIFICATIONREQUEST']._serialized_start=8384
  _globals['_APPLYFORTIFICATIONREQUEST']._serialized_end=8451
  _globals['_APPLYFORTIFICATIONRESPONSE']._serialized_start=8453
  _globals['_APPLYFORTIFICATIONRESPONSE']._serialized_end=8554
  _globals['_SECURITYSCORERESPONSE']._serialized_start=8557
  _globals['_SECURITYSCORERESPONSE']._serialized_end=8774
  _globals['_SECURITYSCORERESPONSE_CATEGORYSCORESENTRY']._serialized_start=8721
  _globals['_SECURITYSCORERESPONSE_CATEGORYSCORESENTRY']._serialized_end=8774
  _globals['_SUPPRESSION']._serialized_start=8777
  _globals['_SUPPRESSION']._serialized_end=8961
  _globals['_ADDSUPPRESSIONREQUEST']._serialized_start=8964
  _globals['_ADDSUPPRESSIONREQUEST']._serialized_end=9126
  _globals['_ADDSUPPRESSIONRESPONSE']._serialized_start=9128
  _globals['_ADDSUPPRESSIONRESPONSE']._serialized_end=9234
  _globals['_REMOVESUPPRESSIONREQUEST']._serialized_start=9236
  _globals['_REMOVESUPPRESSIONREQUEST']._serialized_end=9286
  _globals['_REMOVESUPPRESSIONRESPONSE']._serialized_start=9288
  _globals['_REMOVESUPPRESSIONRESPONSE']._serialized_end=9355
  _globals['_SUPPRESSIONLIST']._serialized_start=9357
  _globals['_SUPPRESSIONLIST']._serialized_end=9438
  _globals['_CHECKSUPPRESSIONREQUEST']._serialized_start=9440
  _globals['_CHECKSUPPRESSIONREQUEST']._serialized_end=9522
  _globals['_CHECKSUPPRESSIONRESPONSE']._serialized_start=9524
  _globals['_CHECKSUPPRESSIONRESPONSE']._serialized_end=9631
  _globals['_UPDATECONFIGREQUEST']._serialized_start=9634
  _globals['_UPDATECONFIGREQUEST']._serialized_end=9765
  _globals['_UPDATECONFIGREQUEST_SETTINGSENTRY']._serialized_start=9718
  _globals['_UPDATECONFIGREQUEST_SETTINGSENTRY']._serialized_end=9765
  _globals['_UPDATECONFIGRESPONSE']._serialized_start=9767
  _globals['_UPDATECONFIGRESPONSE']._serialized_end=9829
  _globals['_UPDATEFRAMESTATUSREQUEST']._serialized_start=9831
  _globals['_UPDATEFRAMESTATUSREQUEST']._serialized_end=9892
  _globals['_UPDATEFRAMESTATUSRESPONSE']._serialized_start=9894
  _globals['_UPDATEFRAMESTATUSRESPONSE']._serialized_end=9991
  _globals['_CLASSIFYREQUEST']._serialized_start=9993
  _globals['_CLASSIFYREQUEST']._serialized_end=10043
  _globals['_CLASSIFYRESULT']._serialized_start=10046
  _globals['_CLASSIFYRESULT']._serialized_end=10336
  _globals['_FRAME']._serialized_start=10338
  _globals['_FRAME']._serialized_end=10461
  _globals['_FRAMELIST']._serialized_start=10463
  _globals['_FRAMELIST']._serialized_end=10505
  _globals['_PROVIDER']._serialized_start=10507
  _globals['_PROVIDER']._serialized_end=10598
  _globals['_PROVIDERLIST']._serialized_start=10600
  _globals['_PROVIDERLIST']._serialized_end=10677
  _globals['_CONFIGURATIONRESPONSE']._serialized_start=10680
  _globals['_CONFIGURATIONRESPONSE']._serialized_end=10978
  _globals['_CONFIGURATIONRESPONSE_SETTINGSENTRY']._serialized_start=9718
  _globals['_CONFIGURATIONRESPONSE_SETTINGSENTRY']._serialized_end=9765
  _globals['_HEALTHRESPONSE']._serialized_start=10981
  _globals['_HEALTHRESPONSE']._serialized_end=11166
  _globals['_HEALTHRESPONSE_COMPONENTSENTRY']._serialized_start=11117
  _globals['_HEALTHRESPONSE_COMPONENTSENTRY']._serialized_end=11166
  _globals['_STATUSRESPONSE']._serialized_start=11169
  _globals['_STATUSRESPONSE']._serialized_end=11313
  _globals['_PIPELINEREQUEST']._serialized_start=11316
  _globals['_PIPELINEREQUEST']._serialized_end=11509
  _globals['_PIPELINEREQUEST_OPTIONSENTRY']._serialized_start=11463
  _globals['_PIPELINEREQUEST_OPTIONSENTRY']._serialized_end=11509
  _globals['_PIPELINERESULT']._serialized_start=11512
  _globals['_PIPELINERESULT']._serialized_end=11858
  _globals['_PIPELINEEVENT']._serialized_start=11861
  _globals['_PIPELINEEVENT']._serialized_end=12084
  _globals['_WARDENSERVICE']._serialized_start=12557
  _globals['_WARDENSERVICE']._serialized_end=16049
# @@protoc_insertion_point(module_scope)



"""
Warden Runtime Services
======================

This package contains entry points for running Warden as a service.
- ipc_entry: Launches the IPC server for CLI communication
- grpc_entry: Launches the gRPC server for external clients
"""



"""
Suppression Mixin

Endpoints: AddSuppression, RemoveSuppression, GetSuppressions, CheckSuppression

Uses repository pattern for persistent storage.
"""

import uuid
from datetime import datetime
from typing import TYPE_CHECKING

try:
    from warden.grpc.generated import warden_pb2
except ImportError:
    warden_pb2 = None

from warden.grpc.converters import ProtoConverters

if TYPE_CHECKING:
    from warden.shared.domain.repository import ISuppressionRepository

try:
    from warden.shared.infrastructure.logging import get_logger

    logger = get_logger(__name__)
except ImportError:
    import logging

    logger = logging.getLogger(__name__)


class SuppressionMixin:
    """Suppression endpoints (4 endpoints) with repository persistence."""

    async def AddSuppression(
        self, request, context
    ) -> "warden_pb2.AddSuppressionResponse":
        """Add a suppression rule with persistence."""
        logger.info("grpc_add_suppression", rule_id=request.rule_id)

        try:
            suppression_id = str(uuid.uuid4())

            suppression = {
                "id": suppression_id,
                "rule_id": request.rule_id,
                "file_path": request.file_path,
                "line_number": request.line_number,
                "justification": request.justification,
                "created_by": request.created_by,
                "created_at": datetime.now().isoformat(),
                "expires_at": request.expires_at or None,
                "is_global": request.is_global,
                "enabled": True,
            }

            # Save to in-memory cache
            self._suppressions[suppression_id] = suppression

            # Persist to repository
            await self.suppression_repository.save(suppression)

            return warden_pb2.AddSuppressionResponse(
                success=True,
                suppression=ProtoConverters.convert_suppression(suppression),
            )

        except Exception as e:
            logger.error("grpc_add_suppression_error: %s", str(e))
            return warden_pb2.AddSuppressionResponse(
                success=False, error_message=str(e)
            )

    async def RemoveSuppression(
        self, request, context
    ) -> "warden_pb2.RemoveSuppressionResponse":
        """Remove a suppression rule with persistence."""
        logger.info("grpc_remove_suppression", suppression_id=request.suppression_id)

        try:
            if request.suppression_id not in self._suppressions:
                return warden_pb2.RemoveSuppressionResponse(
                    success=False,
                    error_message=f"Suppression {request.suppression_id} not found",
                )

            # Remove from in-memory cache
            del self._suppressions[request.suppression_id]

            # Delete from repository
            await self.suppression_repository.delete(request.suppression_id)

            return warden_pb2.RemoveSuppressionResponse(success=True)

        except Exception as e:
            logger.error("grpc_remove_suppression_error: %s", str(e))
            return warden_pb2.RemoveSuppressionResponse(
                success=False, error_message=str(e)
            )

    async def GetSuppressions(self, request, context) -> "warden_pb2.SuppressionList":
        """Get all suppression rules."""
        logger.info("grpc_get_suppressions")

        try:
            response = warden_pb2.SuppressionList(
                total_count=len(self._suppressions)
            )

            for suppression in self._suppressions.values():
                response.suppressions.append(
                    ProtoConverters.convert_suppression(suppression)
                )

            return response

        except Exception as e:
            logger.error("grpc_get_suppressions_error: %s", str(e))
            return warden_pb2.SuppressionList()

    async def CheckSuppression(
        self, request, context
    ) -> "warden_pb2.CheckSuppressionResponse":
        """Check if an issue is suppressed."""
        logger.info("grpc_check_suppression", rule_id=request.rule_id)

        try:
            for suppression in self._suppressions.values():
                if (suppression.get("is_global") and
                        suppression.get("rule_id") == request.rule_id):
                    return warden_pb2.CheckSuppressionResponse(
                        is_suppressed=True,
                        suppression=ProtoConverters.convert_suppression(suppression),
                        reason="Global suppression rule"
                    )

                if (suppression.get("rule_id") == request.rule_id and
                        suppression.get("file_path") == request.file_path):
                    if (suppression.get("line_number", 0) == 0 or
                            suppression.get("line_number") == request.line_number):
                        return warden_pb2.CheckSuppressionResponse(
                            is_suppressed=True,
                            suppression=ProtoConverters.convert_suppression(suppression),
                            reason="File-specific suppression rule"
                        )

            return warden_pb2.CheckSuppressionResponse(
                is_suppressed=False,
                reason="No matching suppression rule"
            )

        except Exception as e:
            logger.error("grpc_check_suppression_error: %s", str(e))
            return warden_pb2.CheckSuppressionResponse(
                is_suppressed=False,
                reason=str(e)
            )



import os
from pathlib import Path
import re

def fix_imports():
    frames_dir = Path(".warden/frames")
    
    if not frames_dir.exists():
        print(f"Error: {frames_dir} does not exist.")
        return

    print(f"Fixing imports in {frames_dir}...")
    
    for frame_path in frames_dir.iterdir():
        if not frame_path.is_dir():
            continue
            
        frame_name = frame_path.name
        
        # Walk through all python files in this frame dir
        for root, dirs, files in os.walk(frame_path):
            for file in files:
                if not file.endswith(".py"):
                    continue
                    
                file_path = Path(root) / file
                
                with open(file_path, "r", encoding="utf-8") as f:
                    content = f.read()
                
                # Regex to find imports from this frame's original package
                # from warden.validation.frames.<frame_name> import ...
                # should become: from . import ...
                
                # Case 1: Import submodule
                # from warden.validation.frames.security._internal.sql... import ...
                # -> from ._internal.sql... import ...
                
                # Careful: 'from .' works if we are importing from a sibling or child.
                # If we are in 'frame.py', 'from ._internal' is correct.
                
                pattern = f"from warden\\.validation\\.frames\\.{frame_name}"
                
                if re.search(pattern, content):
                    print(f"  Fixing {file_path}")
                    new_content = re.sub(pattern, "from .", content)
                    
                    # Fix double dots if happened (unlikely with this regex but be safe)
                    # from .._internal -> from ._internal (if regex matched trailing dot)
                    
                    with open(file_path, "w", encoding="utf-8") as f:
                        f.write(new_content)

    print("Import fix complete.")

if __name__ == "__main__":
    fix_imports()



#!/usr/bin/env python3
"""HTTP Server wrapper for Warden CLI Bridge"""

import asyncio
import json
import os
from pathlib import Path
from dotenv import load_dotenv
from aiohttp import web
from .bridge import WardenBridge
import structlog

import structlog

# Verify credentials using SecretManager (environment-aware)
# This respects the environment: .env for local, env vars for CI/CD, Key Vault for production
async def verify_credentials():
    """Verify Azure OpenAI credentials are available using SecretManager."""
    from warden.secrets import SecretManager
    
    manager = SecretManager()
    
    secrets = await manager.get_secrets([
        "AZURE_OPENAI_API_KEY",
        "AZURE_OPENAI_ENDPOINT",
    ])
    
    azure_key = secrets["AZURE_OPENAI_API_KEY"]
    azure_endpoint = secrets["AZURE_OPENAI_ENDPOINT"]
    
    if azure_key.found:
        print(f"✅ Azure OpenAI credentials loaded successfully")
        print(f"   Source: {azure_key.source.value}")
        print(f"   Endpoint: {azure_endpoint.value if azure_endpoint.found else 'not set'}")
        print(f"   Key: {azure_key.value[:10]}...")
    else:
        print("⚠️ Azure OpenAI credentials not found")
        print(f"   Checked sources: {[p.__class__.__name__ for p in manager.providers]}")

# Run verification
import asyncio
asyncio.run(verify_credentials())

logger = structlog.get_logger()

class HTTPServer:
    def __init__(self):
        self.bridge = WardenBridge()
        self.app = web.Application()
        self.setup_routes()

    def setup_routes(self):
        """Setup HTTP routes"""
        self.app.router.add_post('/rpc', self.handle_rpc)
        self.app.router.add_get('/health', self.handle_health)

    async def handle_health(self, request):
        """Health check endpoint"""
        return web.json_response({
            "status": "healthy",
            "service": "warden-backend",
            "version": "2.0.0"
        })

    async def handle_rpc(self, request):
        """Handle JSON-RPC requests"""
        try:
            data = await request.json()
            method = data.get('method')
            params = data.get('params', {})
            request_id = data.get('id')

            logger.info("rpc_request_received", method=method, params=params)

            # Handle different methods
            if method == 'scan':
                result = await self.handle_scan(params)
            elif method == 'get_config':
                result = await self.handle_get_config(params)
            elif method == 'execute_pipeline_stream':
                # Handle streaming via SSE
                return await self.handle_stream(request, params)
            else:
                return web.json_response({
                    "jsonrpc": "2.0",
                    "error": {
                        "code": -32601,
                        "message": f"Method not found: {method}"
                    },
                    "id": request_id
                })

            return web.json_response({
                "jsonrpc": "2.0",
                "result": result,
                "id": request_id
            })

        except Exception as e:
            logger.error("rpc_error", error=str(e))
            return web.json_response({
                "jsonrpc": "2.0",
                "error": {
                    "code": -32603,
                    "message": str(e)
                },
                "id": data.get('id')
            })

    async def handle_stream(self, request, params):
        """Handle streaming pipeline execution via Server-Sent Events"""
        response = web.StreamResponse(
            status=200,
            reason='OK',
            headers={
                'Content-Type': 'text/event-stream',
                'Cache-Control': 'no-cache',
                'Connection': 'keep-alive',
                'Access-Control-Allow-Origin': '*',
            }
        )
        await response.prepare(request)

        path = params.get('path')
        if not path:
            error_msg = json.dumps({'error': 'Path is required'})
            await response.write(f"event: error\ndata: {error_msg}\n\n".encode('utf-8'))
            return response

        # Resolve path
        path = Path(path).resolve()
        if not path.exists():
            # Try relative to project root
            project_root = Path.cwd()
            path = project_root / params.get('path')
            if not path.exists():
                error_msg = json.dumps({'error': f'File not found: {params.get("path")}'})
                await response.write(f"event: error\ndata: {error_msg}\n\n".encode('utf-8'))
                return response

        logger.info("streaming_pipeline", path=str(path))

        # Prepare frames list
        frames = params.get('frames')

        try:
            # Execute pipeline with streaming
            async for event in self.bridge.execute_pipeline_stream(str(path), frames=frames):
                # Send SSE event
                event_type = event.get('type', 'progress')
                event_data = json.dumps(event)
                await response.write(f"event: {event_type}\ndata: {event_data}\n\n".encode('utf-8'))
                await response.drain()  # Ensure data is sent immediately

            # Send complete event
            await response.write(b"event: complete\ndata: {\"status\": \"completed\"}\n\n")

        except Exception as e:
            logger.error("stream_error", error=str(e))
            error_data = json.dumps({"error": str(e)})
            await response.write(f"event: error\ndata: {error_data}\n\n".encode('utf-8'))

        finally:
            await response.write_eof()

        return response

    async def handle_scan(self, params):
        """Handle scan request"""
        path = params.get('path')
        frames = params.get('frames')

        # Resolve path
        if not path:
            raise ValueError("Path is required")

        path = Path(path).resolve()
        if not path.exists():
            # Try relative to project root
            project_root = Path.cwd()
            path = project_root / params.get('path')
            if not path.exists():
                raise FileNotFoundError(f"File not found: {params.get('path')}")

        logger.info("scanning_file", path=str(path), frames=frames)

        # Perform scan
        result = await self.bridge.scan(str(path), frames=frames)

        return result

    async def handle_get_config(self, params):
        """Handle get_config request"""
        try:
            config = await self.bridge.get_config()

            # Extract available frames
            frames_available = []
            if config and 'frames' in config:
                # Get all configured frames
                for frame in config.get('frames', []):
                    frames_available.append(frame)

            # Also check frames_config for enabled frames
            frames_config = config.get('frames_config', {})
            for frame_id, frame_cfg in frames_config.items():
                if frame_cfg.get('enabled', False) and frame_id not in frames_available:
                    frames_available.append(frame_id)

            return {
                "config": config,
                "frames_available": frames_available,
                "project": config.get('project', {}),
                "llm": config.get('llm', {})
            }
        except Exception as e:
            logger.error("config_error", error=str(e))
            return {
                "config": {},
                "frames_available": ['security', 'chaos', 'orphan'],
                "error": str(e)
            }

    async def start(self, host='localhost', port=6173):
        """Start the HTTP server"""
        logger.info("http_server_starting", host=host, port=port)

        # Bridge is already initialized in __init__

        # Start HTTP server
        runner = web.AppRunner(self.app)
        await runner.setup()
        site = web.TCPSite(runner, host, port)
        await site.start()

        logger.info("http_server_started", url=f"http://{host}:{port}")

        # Keep server running
        try:
            await asyncio.Future()  # Run forever
        except KeyboardInterrupt:
            logger.info("http_server_stopping")
            await runner.cleanup()

async def main():
    """Main entry point"""
    server = HTTPServer()
    await server.start()

if __name__ == "__main__":
    # Configure logging
    structlog.configure(
        processors=[
            structlog.stdlib.filter_by_level,
            structlog.stdlib.add_logger_name,
            structlog.stdlib.add_log_level,
            structlog.stdlib.PositionalArgumentsFormatter(),
            structlog.processors.TimeStamper(fmt="iso"),
            structlog.processors.StackInfoRenderer(),
            structlog.processors.format_exc_info,
            structlog.dev.ConsoleRenderer()
        ],
        context_class=dict,
        logger_factory=structlog.stdlib.LoggerFactory(),
        cache_logger_on_first_use=True,
    )

    asyncio.run(main())


"""
Security Analysis Frame

Detects security vulnerabilities:
- SQL injection
- XSS (Cross-Site Scripting)
- Hardcoded secrets (API keys, passwords)
- Command injection
- Path traversal

Components:
- SecurityFrame: Main frame
- Internal checks: hardcoded_password, secrets, sql_injection, xss

Usage:
    from . import SecurityFrame

    frame = SecurityFrame()
    result = await frame.execute(code_file)
"""

from ..security_frame import SecurityFrame

__all__ = ["SecurityFrame"]



"""
Bridge Utilities - Serialization and Metadata helpers.
"""

from pathlib import Path
from typing import Any, Dict

def detect_language(path: Path) -> str:
    """Detect programming language from file extension."""
    from warden.shared.utils.language_utils import get_language_from_path
    return get_language_from_path(path).value

def serialize_pipeline_result(result: Any) -> Dict[str, Any]:
    """Serialize pipeline result to JSON-RPC compatible dict."""
    if hasattr(result, "to_json"):
        return result.to_json()
    if hasattr(result, "model_dump"):
        return result.model_dump(mode="json")

    # Fallback
    return {
        "pipeline_id": getattr(result, "pipeline_id", "unknown"),
        "status": getattr(result, "status", "unknown").value if hasattr(getattr(result, "status", None), 'value') else str(getattr(result, "status", "unknown")),
        "duration": getattr(result, "duration", 0),
        "total_findings": getattr(result, "total_findings", 0),
        "frame_results": [
            {
                "frame_id": fr.frame_id,
                "status": fr.status.value if hasattr(fr.status, 'value') else str(fr.status),
                "findings": [
                    {
                        "severity": getattr(f, "severity", "unknown"),
                        "message": getattr(f, "message", str(f)),
                        "line": getattr(f, "line_number", getattr(f, "line", 0)),
                    } for f in fr.findings
                ]
            } for fr in getattr(result, "frame_results", [])
        ]
    }



"""
Testability Analyzer

Analyzes code testability and test coverage potential:
- Test file detection and coverage estimation
- Mockability analysis
- Dependency injection patterns
- Side effect detection
- Test complexity assessment
"""

import ast
import re
import structlog
from typing import List, Optional, Dict, Any, Set
from pathlib import Path

from warden.cleaning.domain.base import BaseCleaningAnalyzer, CleaningAnalyzerPriority
from warden.cleaning.domain.models import (
    CleaningResult,
    CleaningSuggestion,
    CleaningIssue,
    CleaningIssueType,
    CleaningIssueSeverity,
)
from warden.validation.domain.frame import CodeFile

logger = structlog.get_logger()


class TestabilityAnalyzer(BaseCleaningAnalyzer):
    """
    Analyzer for code testability metrics.

    Checks:
    - Test coverage potential
    - Mockability of dependencies
    - Dependency injection usage
    - Side effects in functions
    - Test complexity factors
    """

    @property
    def name(self) -> str:
        """Analyzer name."""
        return "Testability Analyzer"

    @property
    def priority(self) -> int:
        """Execution priority."""
        return CleaningAnalyzerPriority.MEDIUM

    async def analyze_async(
        self,
        code_file: CodeFile,
        cancellation_token: Optional[str] = None,
        ast_tree: Optional[Any] = None,
    ) -> CleaningResult:
        """
        Analyze code testability.

        Args:
            code_file: The code file to analyze
            cancellation_token: Optional cancellation token
            ast_tree: Optional pre-parsed AST tree of the code file

        Returns:
            CleaningResult with testability metrics
        """
        if not code_file or not code_file.content:
            return CleaningResult(
                success=False,
                file_path="",
                issues_found=0,
                error_message="Code file is empty",
                analyzer_name=self.name,
            )

        try:
            # Check if this is a test file
            is_test_file = self._is_test_file(code_file.path)

            if is_test_file:
                # Analyze test quality instead of testability
                test_quality = self._analyze_test_quality(code_file.content, ast_tree)
                quality_score = test_quality.get("quality_score", 5.0)
                issues = test_quality.get("issues", [])
            else:
                # Analyze testability
                testability_metrics = self._analyze_testability(code_file.content, ast_tree)
                issues = testability_metrics.get("issues", [])
                quality_score = testability_metrics.get("testability_score", 5.0)

            logger.info(
                "testability_analysis_complete",
                file_path=code_file.path,
                is_test_file=is_test_file,
                quality_score=quality_score,
                issues_found=len(issues),
            )

            # Convert issues to suggestions
            suggestions = [self._create_suggestion(issue, code_file.content) for issue in issues]

            # Calculate cleanup score
            total_lines = len(code_file.content.split("\n"))
            cleanup_score = self._calculate_cleanup_score(len(issues), total_lines)

            summary = (
                f"Test quality score: {quality_score:.1f}/10"
                if is_test_file
                else f"Testability score: {quality_score:.1f}/10"
            )

            return CleaningResult(
                success=True,
                file_path=code_file.path,
                issues_found=len(issues),
                suggestions=suggestions,
                cleanup_score=cleanup_score,
                summary=summary,
                analyzer_name=self.name,
                metrics={
                    "testability_score": quality_score,
                    "is_test_file": is_test_file,
                },
            )

        except Exception as e:
            logger.error(
                "testability_analysis_failed",
                error=str(e),
                file_path=code_file.path,
            )
            return CleaningResult(
                success=False,
                file_path=code_file.path,
                issues_found=0,
                error_message=f"Analysis failed: {str(e)}",
                analyzer_name=self.name,
            )

    def _is_test_file(self, file_path: str) -> bool:
        """Check if file is a test file based on path and naming."""
        path = Path(file_path)
        name_lower = path.name.lower()

        # Check file name patterns
        test_patterns = [
            'test_', '_test.py', 'tests.py',
            'spec_', '_spec.py', 'specs.py'
        ]

        for pattern in test_patterns:
            if pattern in name_lower:
                return True

        # Check if in test directory
        path_parts = [p.lower() for p in path.parts]
        test_dirs = ['test', 'tests', 'spec', 'specs', '__tests__']

        for test_dir in test_dirs:
            if test_dir in path_parts:
                return True

        return False

    def _analyze_testability(self, code: str, ast_tree: Optional[Any] = None) -> Dict[str, Any]:
        """
        Analyze code testability factors.

        Returns:
            Dictionary with testability metrics and issues
        """
        try:
            tree = ast_tree if ast_tree else ast.parse(code)
        except SyntaxError:
            return {"testability_score": 0, "issues": []}

        issues = []
        metrics = {
            "functions": 0,
            "testable_functions": 0,
            "classes": 0,
            "testable_classes": 0,
            "global_state_usage": 0,
            "side_effects": 0,
            "hard_dependencies": 0,
        }

        # Analyze functions
        for node in ast.walk(tree):
            if isinstance(node, ast.FunctionDef):
                metrics["functions"] += 1
                func_issues = self._analyze_function_testability(node, tree)
                if not func_issues:
                    metrics["testable_functions"] += 1
                issues.extend(func_issues)

            elif isinstance(node, ast.ClassDef):
                metrics["classes"] += 1
                class_issues = self._analyze_class_testability(node)
                if not class_issues:
                    metrics["testable_classes"] += 1
                issues.extend(class_issues)

        # Detect global state usage
        global_issues = self._detect_global_state_usage(tree)
        metrics["global_state_usage"] = len(global_issues)
        issues.extend(global_issues)

        # Calculate testability score (0-10)
        score = 10.0

        # Deduct for untestable functions
        if metrics["functions"] > 0:
            testable_ratio = metrics["testable_functions"] / metrics["functions"]
            score -= (1 - testable_ratio) * 3

        # Deduct for untestable classes
        if metrics["classes"] > 0:
            testable_ratio = metrics["testable_classes"] / metrics["classes"]
            score -= (1 - testable_ratio) * 2

        # Deduct for global state usage
        score -= min(metrics["global_state_usage"] * 0.5, 3)

        # Ensure score is within bounds
        score = max(0, min(10, score))

        return {
            "testability_score": score,
            "issues": issues,
            "metrics": metrics,
        }

    def _analyze_function_testability(self, node: ast.FunctionDef, tree: ast.AST) -> List[CleaningIssue]:
        """Analyze testability issues in a function."""
        issues = []

        # Check for side effects
        has_side_effects = self._has_side_effects(node)
        if has_side_effects:
            issues.append(
                CleaningIssue(
                    issue_type=CleaningIssueType.TESTABILITY_ISSUE,
                    description=f"Function '{node.name}' has side effects that make testing difficult",
                    line_number=node.lineno,
                    severity=CleaningIssueSeverity.MEDIUM,
                )
            )

        # Check for hard dependencies
        hard_deps = self._detect_hard_dependencies(node)
        if hard_deps:
            issues.append(
                CleaningIssue(
                    issue_type=CleaningIssueType.TESTABILITY_ISSUE,
                    description=f"Function '{node.name}' has hard dependencies: {', '.join(hard_deps)}",
                    line_number=node.lineno,
                    severity=CleaningIssueSeverity.MEDIUM,
                )
            )

        # Check for complex control flow
        complexity = self._calculate_complexity(node)
        if complexity > 10:
            issues.append(
                CleaningIssue(
                    issue_type=CleaningIssueType.TESTABILITY_ISSUE,
                    description=f"Function '{node.name}' has high complexity ({complexity}) making it hard to test",
                    line_number=node.lineno,
                    severity=CleaningIssueSeverity.HIGH,
                )
            )

        # Check for global variable access
        if self._uses_global_variables(node):
            issues.append(
                CleaningIssue(
                    issue_type=CleaningIssueType.TESTABILITY_ISSUE,
                    description=f"Function '{node.name}' accesses global variables",
                    line_number=node.lineno,
                    severity=CleaningIssueSeverity.MEDIUM,
                )
            )

        return issues

    def _analyze_class_testability(self, node: ast.ClassDef) -> List[CleaningIssue]:
        """Analyze testability issues in a class."""
        issues = []

        # Check for singleton pattern (hard to test)
        if self._is_singleton(node):
            issues.append(
                CleaningIssue(
                    issue_type=CleaningIssueType.TESTABILITY_ISSUE,
                    description=f"Class '{node.name}' uses singleton pattern which is hard to test",
                    line_number=node.lineno,
                    severity=CleaningIssueSeverity.HIGH,
                )
            )

        # Check for static methods that use external resources
        static_issues = self._check_static_methods(node)
        issues.extend(static_issues)

        # Check for tight coupling in __init__
        init_method = self._get_init_method(node)
        if init_method:
            # Check if __init__ creates dependencies instead of receiving them
            if self._creates_dependencies_in_init(init_method):
                issues.append(
                    CleaningIssue(
                        issue_type=CleaningIssueType.TESTABILITY_ISSUE,
                        description=f"Class '{node.name}' creates dependencies in __init__ instead of using dependency injection",
                        line_number=init_method.lineno,
                        severity=CleaningIssueSeverity.MEDIUM,
                    )
                )

        return issues

    def _analyze_test_quality(self, code: str, ast_tree: Optional[Any] = None) -> Dict[str, Any]:
        """
        Analyze test file quality.

        Returns:
            Dictionary with test quality metrics
        """
        try:
            tree = ast_tree if ast_tree else ast.parse(code)
        except SyntaxError:
            return {"quality_score": 0, "issues": []}

        issues = []
        test_count = 0
        assertion_count = 0
        mock_usage = 0

        for node in ast.walk(tree):
            # Count test functions
            if isinstance(node, ast.FunctionDef) and node.name.startswith('test'):
                test_count += 1

                # Check for assertions
                has_assertion = False
                for child in ast.walk(node):
                    if isinstance(child, ast.Assert):
                        has_assertion = True
                        assertion_count += 1
                    elif isinstance(child, ast.Call):
                        if hasattr(child.func, 'attr'):
                            # Check for unittest assertions
                            if child.func.attr.startswith('assert'):
                                has_assertion = True
                                assertion_count += 1

                if not has_assertion:
                    issues.append(
                        CleaningIssue(
                            issue_type=CleaningIssueType.TESTABILITY_ISSUE,
                            description=f"Test '{node.name}' has no assertions",
                            line_number=node.lineno,
                            severity=CleaningIssueSeverity.HIGH,
                        )
                    )

                # Check test complexity
                complexity = self._calculate_complexity(node)
                if complexity > 5:
                    issues.append(
                        CleaningIssue(
                            issue_type=CleaningIssueType.TESTABILITY_ISSUE,
                            description=f"Test '{node.name}' is too complex (complexity: {complexity})",
                            line_number=node.lineno,
                            severity=CleaningIssueSeverity.MEDIUM,
                        )
                    )

        # Calculate quality score
        score = 10.0

        if test_count == 0:
            score = 0
        else:
            # Deduct for tests without assertions
            tests_without_assertions = sum(1 for issue in issues if "no assertions" in issue.description)
            score -= (tests_without_assertions / test_count) * 5

            # Deduct for complex tests
            complex_tests = sum(1 for issue in issues if "too complex" in issue.description)
            score -= (complex_tests / test_count) * 3

        return {
            "quality_score": max(0, min(10, score)),
            "issues": issues,
            "test_count": test_count,
            "assertion_count": assertion_count,
        }

    def _has_side_effects(self, node: ast.FunctionDef) -> bool:
        """Check if function has side effects."""
        for child in ast.walk(node):
            # File I/O
            if isinstance(child, ast.Call):
                if hasattr(child.func, 'id') and child.func.id in ['open', 'print']:
                    return True
                if hasattr(child.func, 'attr') and child.func.attr in ['write', 'read', 'save']:
                    return True

            # Global modifications
            if isinstance(child, ast.Global):
                return True

            # Database/network calls (heuristic)
            if isinstance(child, ast.Attribute):
                if hasattr(child, 'attr'):
                    if any(keyword in child.attr.lower() for keyword in ['save', 'delete', 'update', 'insert', 'fetch', 'post', 'get']):
                        return True

        return False

    def _detect_hard_dependencies(self, node: ast.FunctionDef) -> List[str]:
        """Detect hard-coded dependencies in function."""
        hard_deps = []

        for child in ast.walk(node):
            if isinstance(child, ast.Call):
                # Direct instantiation of external classes
                if hasattr(child.func, 'id'):
                    class_name = child.func.id
                    # Common external dependencies
                    if class_name in ['HTTPClient', 'DatabaseConnection', 'FileManager', 'Logger']:
                        hard_deps.append(class_name)

                # Import and instantiate pattern
                if isinstance(child.func, ast.Attribute):
                    if hasattr(child.func.value, 'id'):
                        module = child.func.value.id
                        if module in ['requests', 'urllib', 'socket', 'subprocess']:
                            hard_deps.append(f"{module}.{child.func.attr}")

        return hard_deps

    def _calculate_complexity(self, node: ast.FunctionDef) -> int:
        """Calculate cyclomatic complexity of a function."""
        complexity = 1

        for child in ast.walk(node):
            if isinstance(child, (ast.If, ast.While, ast.For)):
                complexity += 1
            elif isinstance(child, ast.BoolOp):
                complexity += len(child.values) - 1
            elif isinstance(child, ast.ExceptHandler):
                complexity += 1

        return complexity

    def _uses_global_variables(self, node: ast.FunctionDef) -> bool:
        """Check if function uses global variables."""
        for child in ast.walk(node):
            if isinstance(child, (ast.Global, ast.Nonlocal)):
                return True
        return False

    def _detect_global_state_usage(self, tree: ast.AST) -> List[CleaningIssue]:
        """Detect global state usage in module."""
        issues = []
        global_vars = set()

        # Find global variable assignments
        for node in tree.body:
            if isinstance(node, ast.Assign):
                for target in node.targets:
                    if isinstance(target, ast.Name):
                        if target.id.isupper():  # Constants are OK
                            continue
                        global_vars.add(target.id)

        # Check if globals are modified in functions
        for node in ast.walk(tree):
            if isinstance(node, ast.FunctionDef):
                for child in ast.walk(node):
                    if isinstance(child, ast.Global):
                        for name in child.names:
                            if name in global_vars:
                                issues.append(
                                    CleaningIssue(
                                        issue_type=CleaningIssueType.TESTABILITY_ISSUE,
                                        description=f"Global variable '{name}' modified in function '{node.name}'",
                                        line_number=child.lineno,
                                        severity=CleaningIssueSeverity.HIGH,
                                    )
                                )

        return issues

    def _is_singleton(self, node: ast.ClassDef) -> bool:
        """Check if class implements singleton pattern."""
        has_instance = False
        has_new = False

        for child in node.body:
            if isinstance(child, ast.Assign):
                for target in child.targets:
                    if isinstance(target, ast.Name) and '_instance' in target.id.lower():
                        has_instance = True

            if isinstance(child, ast.FunctionDef):
                if child.name == '__new__':
                    has_new = True

        return has_instance or has_new

    def _check_static_methods(self, node: ast.ClassDef) -> List[CleaningIssue]:
        """Check static methods for testability issues."""
        issues = []

        for child in node.body:
            if isinstance(child, ast.FunctionDef):
                # Check if it's a static method
                is_static = any(
                    isinstance(dec, ast.Name) and dec.id == 'staticmethod'
                    for dec in child.decorator_list
                )

                if is_static and self._has_side_effects(child):
                    issues.append(
                        CleaningIssue(
                            issue_type=CleaningIssueType.TESTABILITY_ISSUE,
                            description=f"Static method '{child.name}' has side effects",
                            line_number=child.lineno,
                            severity=CleaningIssueSeverity.MEDIUM,
                        )
                    )

        return issues

    def _get_init_method(self, node: ast.ClassDef) -> Optional[ast.FunctionDef]:
        """Get __init__ method from class."""
        for child in node.body:
            if isinstance(child, ast.FunctionDef) and child.name == '__init__':
                return child
        return None

    def _creates_dependencies_in_init(self, init_method: ast.FunctionDef) -> bool:
        """Check if __init__ creates dependencies instead of receiving them."""
        for child in ast.walk(init_method):
            if isinstance(child, ast.Call):
                # Check for instantiation of external classes
                if hasattr(child.func, 'id'):
                    class_name = child.func.id
                    # Common dependencies that should be injected
                    if any(keyword in class_name.lower() for keyword in ['client', 'connection', 'manager', 'service', 'repository']):
                        # Check if it's assigned to self
                        parent = child
                        while parent:
                            if isinstance(parent, ast.Assign):
                                for target in parent.targets:
                                    if isinstance(target, ast.Attribute) and isinstance(target.value, ast.Name) and target.value.id == 'self':
                                        return True
                            parent = getattr(parent, 'parent', None)

        return False

    def _create_suggestion(self, issue: CleaningIssue, code: str) -> CleaningSuggestion:
        """Create a cleanup suggestion from an issue."""
        # Extract code snippet
        code_snippet = self._get_code_snippet(code, issue.line_number, context=3)
        issue.code_snippet = code_snippet

        # Generate suggestion based on issue description
        if "side effects" in issue.description:
            suggestion = "Refactor to pure functions or isolate side effects"
            rationale = "Pure functions are easier to test and reason about"
            example_code = """# Separate I/O from logic:
def calculate_result(data):
    # Pure logic
    return processed_data

def save_result(result, file_path):
    # Side effect isolated
    with open(file_path, 'w') as f:
        f.write(result)"""

        elif "hard dependencies" in issue.description:
            suggestion = "Use dependency injection to make code testable"
            rationale = "Dependency injection allows mocking in tests"
            example_code = """# Inject dependencies:
def process_data(data, http_client=None):
    client = http_client or HTTPClient()
    return client.fetch(data)"""

        elif "high complexity" in issue.description:
            suggestion = "Break down complex function into smaller, testable units"
            rationale = "Simple functions are easier to test comprehensively"
            example_code = """# Extract complex logic:
def main_function(data):
    validated = validate_data(data)
    processed = process_step1(validated)
    return process_step2(processed)"""

        elif "global variables" in issue.description:
            suggestion = "Pass state as parameters instead of using globals"
            rationale = "Functions without global state are predictable and testable"
            example_code = """# Pass state explicitly:
def process(data, config):
    # Use config parameter instead of global
    return apply_config(data, config)"""

        elif "singleton pattern" in issue.description:
            suggestion = "Consider using dependency injection instead of singleton"
            rationale = "Singletons make testing difficult due to shared state"
            example_code = """# Use dependency injection:
class Service:
    def __init__(self, config):
        self.config = config"""

        elif "no assertions" in issue.description:
            suggestion = "Add assertions to verify test expectations"
            rationale = "Tests without assertions don't verify behavior"
            example_code = """def test_addition():
    result = add(2, 3)
    assert result == 5
    assert isinstance(result, int)"""

        elif "too complex" in issue.description:
            suggestion = "Simplify test by focusing on single behavior"
            rationale = "Simple tests are easier to understand and maintain"
            example_code = """# One test per behavior:
def test_valid_input():
    assert process("valid") == "result"

def test_invalid_input():
    with pytest.raises(ValueError):
        process("invalid")"""

        else:
            suggestion = "Improve code testability"
            rationale = "Testable code is more maintainable and reliable"
            example_code = ""

        return CleaningSuggestion(
            issue=issue,
            suggestion=suggestion,
            rationale=rationale,
            example_code=example_code,
        )


"""Pipeline domain package."""

from warden.pipeline.domain.models import (
    ValidationPipeline,
    PipelineConfig,
    PipelineResult,
    FrameExecution,
)
from warden.pipeline.domain.enums import PipelineStatus, ExecutionStrategy

__all__ = [
    "ValidationPipeline",
    "PipelineConfig",
    "PipelineResult",
    "FrameExecution",
    "PipelineStatus",
    "ExecutionStrategy",
]



"""
Tests for build_context.models module.

Tests Panel JSON compatibility and model functionality.
"""

import pytest
from warden.build_context.models import (
    BuildSystem,
    DependencyType,
    Dependency,
    BuildScript,
    BuildContext,
    create_empty_context,
)


class TestBuildSystem:
    """Test BuildSystem enum."""

    def test_enum_values(self) -> None:
        """Test enum has expected values."""
        assert BuildSystem.UNKNOWN.value == 0
        assert BuildSystem.NPM.value == 1
        assert BuildSystem.YARN.value == 2
        assert BuildSystem.POETRY.value == 5
        assert BuildSystem.PIP.value == 4

    def test_enum_serialization(self) -> None:
        """Test enum can be serialized to int."""
        assert BuildSystem.NPM.value == 1
        assert BuildSystem.POETRY.value == 5


class TestDependencyType:
    """Test DependencyType enum."""

    def test_enum_values(self) -> None:
        """Test enum has expected values."""
        assert DependencyType.PRODUCTION.value == 0
        assert DependencyType.DEVELOPMENT.value == 1
        assert DependencyType.OPTIONAL.value == 2
        assert DependencyType.PEER.value == 3


class TestDependency:
    """Test Dependency model."""

    def test_create_simple_dependency(self) -> None:
        """Test creating simple dependency."""
        dep = Dependency(
            name="fastapi",
            version=">=0.100.0",
            type=DependencyType.PRODUCTION,
            is_direct=True
        )

        assert dep.name == "fastapi"
        assert dep.version == ">=0.100.0"
        assert dep.type == DependencyType.PRODUCTION
        assert dep.is_direct is True
        assert dep.extras == []

    def test_create_dependency_with_extras(self) -> None:
        """Test creating dependency with extras."""
        dep = Dependency(
            name="httpx",
            version="0.24.1",
            type=DependencyType.PRODUCTION,
            is_direct=True,
            extras=["http2", "socks"]
        )

        assert dep.name == "httpx"
        assert dep.extras == ["http2", "socks"]

    def test_to_json(self) -> None:
        """Test converting to Panel JSON."""
        dep = Dependency(
            name="react",
            version="^18.2.0",
            type=DependencyType.PRODUCTION,
            is_direct=True
        )

        json_data = dep.to_json()

        assert json_data["name"] == "react"
        assert json_data["version"] == "^18.2.0"
        assert json_data["type"] == 0  # Enum serialized to int
        assert json_data["isDirect"] is True  # camelCase
        assert json_data["extras"] == []


class TestBuildScript:
    """Test BuildScript model."""

    def test_create_script(self) -> None:
        """Test creating build script."""
        script = BuildScript(
            name="build",
            command="tsc",
            description="Compile TypeScript"
        )

        assert script.name == "build"
        assert script.command == "tsc"
        assert script.description == "Compile TypeScript"

    def test_to_json(self) -> None:
        """Test converting to Panel JSON."""
        script = BuildScript(
            name="test",
            command="pytest",
            description="Run tests"
        )

        json_data = script.to_json()

        assert json_data["name"] == "test"
        assert json_data["command"] == "pytest"
        assert json_data["description"] == "Run tests"


class TestBuildContext:
    """Test BuildContext model."""

    def test_create_minimal_context(self) -> None:
        """Test creating minimal build context."""
        context = BuildContext(
            build_system=BuildSystem.NPM,
            project_path="/path/to/project"
        )

        assert context.build_system == BuildSystem.NPM
        assert context.project_path == "/path/to/project"
        assert context.dependencies == []
        assert context.dev_dependencies == []
        assert context.scripts == []

    def test_create_full_context(self) -> None:
        """Test creating full build context."""
        deps = [
            Dependency(name="fastapi", version=">=0.100.0"),
        ]
        dev_deps = [
            Dependency(name="pytest", version="^7.0.0", type=DependencyType.DEVELOPMENT),
        ]
        scripts = [
            BuildScript(name="test", command="pytest"),
        ]

        context = BuildContext(
            build_system=BuildSystem.POETRY,
            project_path="/path/to/project",
            project_name="my-app",
            project_version="1.0.0",
            project_description="My application",
            config_file_path="/path/to/project/pyproject.toml",
            dependencies=deps,
            dev_dependencies=dev_deps,
            scripts=scripts,
            python_version="^3.11",
        )

        assert context.build_system == BuildSystem.POETRY
        assert context.project_name == "my-app"
        assert context.project_version == "1.0.0"
        assert len(context.dependencies) == 1
        assert len(context.dev_dependencies) == 1
        assert len(context.scripts) == 1

    def test_get_all_dependencies(self) -> None:
        """Test getting all dependencies."""
        context = BuildContext(
            build_system=BuildSystem.NPM,
            project_path="/path/to/project",
            dependencies=[
                Dependency(name="react", version="^18.0.0"),
            ],
            dev_dependencies=[
                Dependency(name="typescript", version="^5.0.0"),
            ]
        )

        all_deps = context.get_all_dependencies()

        assert len(all_deps) == 2
        assert all_deps[0].name == "react"
        assert all_deps[1].name == "typescript"

    def test_get_dependency_by_name(self) -> None:
        """Test finding dependency by name."""
        context = BuildContext(
            build_system=BuildSystem.NPM,
            project_path="/path/to/project",
            dependencies=[
                Dependency(name="react", version="^18.0.0"),
                Dependency(name="vue", version="^3.0.0"),
            ]
        )

        dep = context.get_dependency_by_name("react")
        assert dep is not None
        assert dep.name == "react"

        # Test case-insensitive
        dep = context.get_dependency_by_name("REACT")
        assert dep is not None

        # Test not found
        dep = context.get_dependency_by_name("angular")
        assert dep is None

    def test_has_dependency(self) -> None:
        """Test checking if dependency exists."""
        context = BuildContext(
            build_system=BuildSystem.PIP,
            project_path="/path/to/project",
            dependencies=[
                Dependency(name="fastapi", version=">=0.100.0"),
            ]
        )

        assert context.has_dependency("fastapi") is True
        assert context.has_dependency("FASTAPI") is True  # Case-insensitive
        assert context.has_dependency("django") is False

    def test_get_production_dependencies(self) -> None:
        """Test getting only production dependencies."""
        context = BuildContext(
            build_system=BuildSystem.NPM,
            project_path="/path/to/project",
            dependencies=[
                Dependency(name="react", version="^18.0.0", type=DependencyType.PRODUCTION),
                Dependency(name="lodash", version="^4.0.0", type=DependencyType.OPTIONAL),
            ],
            dev_dependencies=[
                Dependency(name="typescript", version="^5.0.0", type=DependencyType.DEVELOPMENT),
            ]
        )

        prod_deps = context.get_production_dependencies()

        assert len(prod_deps) == 1
        assert prod_deps[0].name == "react"

    def test_get_script_by_name(self) -> None:
        """Test finding script by name."""
        context = BuildContext(
            build_system=BuildSystem.NPM,
            project_path="/path/to/project",
            scripts=[
                BuildScript(name="build", command="tsc"),
                BuildScript(name="test", command="jest"),
            ]
        )

        script = context.get_script_by_name("build")
        assert script is not None
        assert script.command == "tsc"

        script = context.get_script_by_name("deploy")
        assert script is None

    def test_has_script(self) -> None:
        """Test checking if script exists."""
        context = BuildContext(
            build_system=BuildSystem.NPM,
            project_path="/path/to/project",
            scripts=[
                BuildScript(name="build", command="tsc"),
            ]
        )

        assert context.has_script("build") is True
        assert context.has_script("test") is False

    def test_to_json(self) -> None:
        """Test converting to Panel JSON."""
        context = BuildContext(
            build_system=BuildSystem.NPM,
            project_path="/path/to/project",
            project_name="my-app",
            project_version="1.0.0",
            dependencies=[
                Dependency(name="react", version="^18.0.0"),
            ],
            scripts=[
                BuildScript(name="build", command="tsc"),
            ]
        )

        json_data = context.to_json()

        # Check camelCase conversion
        assert json_data["buildSystem"] == 1  # NPM enum value
        assert json_data["projectPath"] == "/path/to/project"
        assert json_data["projectName"] == "my-app"
        assert json_data["projectVersion"] == "1.0.0"

        # Check nested objects
        assert len(json_data["dependencies"]) == 1
        assert json_data["dependencies"][0]["name"] == "react"
        assert len(json_data["scripts"]) == 1
        assert json_data["scripts"][0]["name"] == "build"


class TestCreateEmptyContext:
    """Test create_empty_context helper function."""

    def test_create_empty_context(self) -> None:
        """Test creating empty context."""
        context = create_empty_context("/path/to/project")

        assert context.build_system == BuildSystem.UNKNOWN
        assert context.project_path == "/path/to/project"
        assert context.project_name == "Unknown Project"
        assert context.project_version == "0.0.0"
        assert context.dependencies == []
        assert context.dev_dependencies == []
